from typing import Union

from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase


class DispPolylineEndTypeEnum:
    None_ = 0
    Arrow = 1
    Dot = 2


class ShapePolyLine(ShapeBase):
    @staticmethod
    def targetGroupId(disp) -> int:
        return disp.get("ti")

    @staticmethod
    def targetEdgeTemplateCoordSetId(disp) -> Union[int, None]:
        if disp.get("tn") is None or "|" not in disp.get("tn"):
            return None

        id_, _, _ = disp.partition("|")
        return id_

    @staticmethod
    def targetEdgeTemplateName(disp) -> Union[str, None]:
        if disp.get("tn") is None or "|" not in disp.get("tn"):
            return None

        _, _, name = disp.partition("|")
        return name

    @staticmethod
    def edgeColor(disp) -> DispColor:
        return disp.get("ecl")

    @staticmethod
    def startKey(disp) -> Union[str, None]:
        return disp.get("sk")

    @staticmethod
    def startEndType(disp) -> int:
        return disp.get("st") if disp.get("st") else 0

    @staticmethod
    def endEndType(disp) -> int:
        return disp.get("et") if disp.get("et") else 0

    @staticmethod
    def center(disp) -> Point:
        # TODO
        raise NotImplementedError

    @staticmethod
    def firstPoint(disp) -> Point:
        return Point(x=disp.get("g")[0], y=disp.get("g")[1])

    @staticmethod
    def lastPoint(disp) -> Point:
        length = len(disp.get("g"))
        return Point(x=disp.get("g")[length - 2], y=disp.get("g")[length - 1])

    @staticmethod
    def contains(disp, point: Point, margin: float):
        # TODO
        raise NotImplementedError



import json
from datetime import datetime

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple

from peek_plugin_diagram.tuples.grids.DecodedCompiledGridTuple import (
    DecodedCompiledGridTuple,
)


@addTupleType
class GridTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "GridTuple"

    gridKey: str = TupleField()
    dispJsonStr: str = TupleField()
    lastUpdate: datetime = TupleField()

    def toDecodedCompiledGridTuple(self) -> DecodedCompiledGridTuple:
        disps = json.loads(self.dispJsonStr)

        return DecodedCompiledGridTuple(
            gridKey=self.gridKey,
            shapes=disps,
            lastUpdate=self.lastUpdate,
            isLinkedWithLookups=False,
            compiledShapes=[],
        )



from typing import List


from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram._private.storage.Lookups import DispLayer
from peek_plugin_diagram._private.storage.Lookups import DispLevel
from peek_plugin_diagram._private.storage.Lookups import DispLineStyle
from peek_plugin_diagram._private.storage.Lookups import DispTextStyle


class WorkerDiagramLookupApiImpl:
    def __init__(self):
        pass

    @classmethod
    def getColors(cls) -> List[DispColor]:
        ormSession = CeleryDbConn.getDbSession()
        try:
            rows = ormSession.query(DispColor).all()

            tuples = []

            for row in rows:
                tuple_ = row.toTuple()
                tuples.append(tuple_)

            return tuples

        finally:
            ormSession.close()

    @classmethod
    def getLineStyles(cls) -> List[DispLineStyle]:
        ormSession = CeleryDbConn.getDbSession()
        try:
            rows = ormSession.query(DispLineStyle).all()

            tuples = []
            for row in rows:
                tuple_ = row.toTuple()
                tuples.append(tuple_)

            return tuples

        finally:
            ormSession.close()

    @classmethod
    def getTextStyles(cls) -> List[DispTextStyle]:
        ormSession = CeleryDbConn.getDbSession()
        try:
            rows = ormSession.query(DispTextStyle).all()

            tuples = []
            for row in rows:
                tuple_ = row.toTuple()
                tuples.append(tuple_)

            return tuples

        finally:
            ormSession.close()

    @classmethod
    def getLayers(cls) -> List[DispLayer]:
        ormSession = CeleryDbConn.getDbSession()
        try:
            rows = ormSession.query(DispLayer).all()

            tuples = []
            for row in rows:
                tuple_ = row.toTuple()
                tuples.append(tuple_)

            return tuples

        finally:
            ormSession.close()

    @classmethod
    def getLevels(cls) -> List[DispLevel]:
        ormSession = CeleryDbConn.getDbSession()
        try:
            rows = ormSession.query(DispLevel).all()

            tuples = []
            for row in rows:
                tuple_ = row.toTuple()
                tuples.append(tuple_)

            return tuples

        finally:
            ormSession.close()



import logging

from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger

from peek_plugin_base.server.PluginLogicEntryHookABC import (
    PluginLogicEntryHookABC,
)
from peek_plugin_base.server.PluginServerStorageEntryHookABC import (
    PluginServerStorageEntryHookABC,
)
from peek_plugin_base.server.PluginServerWorkerEntryHookABC import (
    PluginServerWorkerEntryHookABC,
)
from peek_plugin_diagram._private.server.api.DiagramApi import DiagramApi
from peek_plugin_diagram._private.server.client_handlers.BranchIndexChunkLoadRpc import (
    BranchIndexChunkLoadRpc,
)
from peek_plugin_diagram._private.server.client_handlers.BranchIndexChunkUpdateHandler import (
    BranchIndexChunkUpdateHandler,
)
from peek_plugin_diagram._private.server.client_handlers.ClientGridLoaderRpc import (
    ClientGridLoaderRpc,
)
from peek_plugin_diagram._private.server.client_handlers.ClientGridUpdateHandler import (
    ClientGridUpdateHandler,
)
from peek_plugin_diagram._private.server.client_handlers.ClientLocationIndexLoaderRpc import (
    ClientLocationIndexLoaderRpc,
)
from peek_plugin_diagram._private.server.client_handlers.ClientLocationIndexUpdateHandler import (
    ClientLocationIndexUpdateHandler,
)
from peek_plugin_diagram._private.server.controller.BranchIndexCompilerQueueController import (
    BranchIndexCompilerQueueController,
)
from peek_plugin_diagram._private.server.controller.BranchLiveEditController import (
    BranchLiveEditController,
)
from peek_plugin_diagram._private.server.controller.BranchUpdateController import (
    BranchUpdateController,
)
from peek_plugin_diagram._private.server.controller.DispCompilerQueueController import (
    DispCompilerQueueController,
)
from peek_plugin_diagram._private.server.controller.DispImportController import (
    DispImportController,
)
from peek_plugin_diagram._private.server.controller.GridKeyCompilerQueueController import (
    GridKeyCompilerQueueController,
)
from peek_plugin_diagram._private.server.controller.LiveDbWatchController import (
    LiveDbWatchController,
)
from peek_plugin_diagram._private.server.controller.LocationCompilerQueueController import (
    LocationCompilerQueueController,
)
from peek_plugin_diagram._private.server.controller.LookupImportController import (
    LookupImportController,
)
from peek_plugin_diagram._private.storage import DeclarativeBase
from peek_plugin_diagram._private.storage.DeclarativeBase import (
    loadStorageTuples,
)
from peek_plugin_diagram._private.storage.Setting import BRANCH_COMPILER_ENABLED
from peek_plugin_diagram._private.storage.Setting import DISP_COMPILER_ENABLED
from peek_plugin_diagram._private.storage.Setting import GRID_COMPILER_ENABLED
from peek_plugin_diagram._private.storage.Setting import (
    LOCATION_COMPILER_ENABLED,
)
from peek_plugin_diagram._private.storage.Setting import globalProperties
from peek_plugin_diagram._private.storage.Setting import globalSetting
from peek_plugin_diagram._private.tuples import loadPrivateTuples
from peek_plugin_diagram.tuples import loadPublicTuples
from peek_plugin_livedb.server.LiveDBApiABC import LiveDBApiABC
from .TupleActionProcessor import makeTupleActionProcessorHandler
from .TupleDataObservable import makeTupleDataObservableHandler
from .admin_handlers import makeAdminBackendHandlers
from .controller.StatusController import StatusController

logger = logging.getLogger(__name__)


class LogicEntryHook(
    PluginLogicEntryHookABC,
    PluginServerStorageEntryHookABC,
    PluginServerWorkerEntryHookABC,
):
    def __init__(self, *args, **kwargs):
        """ " Constructor"""
        # Call the base classes constructor
        PluginLogicEntryHookABC.__init__(self, *args, **kwargs)

        #: Loaded Objects, This is a list of all objects created when we start
        self._loadedObjects = []

        self._api = None

    def load(self) -> None:
        """Load

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialiastion steps here.

        """
        loadStorageTuples()
        loadPrivateTuples()
        loadPublicTuples()
        logger.debug("Loaded")

    @property
    def dbMetadata(self):
        return DeclarativeBase.metadata

    @inlineCallbacks
    def start(self):
        """Start

        This will be called when the plugin is loaded, just after the db is migrated.
        Place any custom initialisation steps here.

        """

        # ----------------
        # Get a reference to the LiveDB API
        liveDbApi: LiveDBApiABC = self.platform.getOtherPluginApi(
            "peek_plugin_livedb"
        )

        # ----------------
        # create the client grid updater
        clientGridUpdateHandler = ClientGridUpdateHandler(self.dbSessionCreator)
        self._loadedObjects.append(clientGridUpdateHandler)

        # ----------------
        # create the client disp key index updater
        clientDispIndexUpdateHandler = ClientLocationIndexUpdateHandler(
            self.dbSessionCreator
        )
        self._loadedObjects.append(clientDispIndexUpdateHandler)

        # ----------------
        # Create the client branch index handler
        clientBranchIndexChunkUpdateHandler = BranchIndexChunkUpdateHandler(
            self.dbSessionCreator
        )
        self._loadedObjects.append(clientBranchIndexChunkUpdateHandler)

        # ----------------
        # create the Status Controller
        statusController = StatusController()
        self._loadedObjects.append(statusController)

        # ----------------
        # Create the GRID KEY queue
        gridKeyCompilerQueueController = GridKeyCompilerQueueController(
            self.dbSessionCreator, statusController, clientGridUpdateHandler
        )
        self._loadedObjects.append(gridKeyCompilerQueueController)

        def locationsCanBeQueuedFunc() -> bool:
            return (
                not gridKeyCompilerQueueController.isBusy()
                and not dispCompilerQueueController.isBusy()
            )

        # ----------------
        # Create the LOCATION INDEX queue
        locationIndexCompilerQueueController = LocationCompilerQueueController(
            self.dbSessionCreator,
            statusController,
            clientDispIndexUpdateHandler,
            readyLambdaFunc=locationsCanBeQueuedFunc,
        )
        self._loadedObjects.append(locationIndexCompilerQueueController)

        # ----------------
        # Create the DISP queue
        dispCompilerQueueController = DispCompilerQueueController(
            self.dbSessionCreator, statusController
        )
        self._loadedObjects.append(dispCompilerQueueController)

        # ----------------
        # Branch Live Edit Controller

        branchLiveEditController = BranchLiveEditController()
        self._loadedObjects.append(branchLiveEditController)

        # ----------------
        # Branch Index Compiler Controller
        branchIndexCompilerQueueController = BranchIndexCompilerQueueController(
            dbSessionCreator=self.dbSessionCreator,
            statusController=statusController,
            clientUpdateHandler=clientBranchIndexChunkUpdateHandler,
        )
        self._loadedObjects.append(branchIndexCompilerQueueController)

        # ----------------
        # Create the Tuple Observer
        tupleObservable = makeTupleDataObservableHandler(
            self.dbSessionCreator, statusController, branchLiveEditController
        )
        self._loadedObjects.append(tupleObservable)

        # ----------------
        # Tell the status controller about the Tuple Observable
        statusController.setTupleObservable(tupleObservable)
        branchLiveEditController.setTupleObservable(tupleObservable)

        # ----------------
        # Initialise the handlers for the admin interface
        self._loadedObjects.extend(
            makeAdminBackendHandlers(tupleObservable, self.dbSessionCreator)
        )

        # ----------------
        # Create the display object Import Controller
        dispImportController = DispImportController(
            dbSessionCreator=self.dbSessionCreator,
            liveDbWriteApi=liveDbApi.writeApi,
        )
        self._loadedObjects.append(dispImportController)

        # ----------------
        # Create the import lookup controller
        lookupImportController = LookupImportController(
            dbSessionCreator=self.dbSessionCreator
        )
        self._loadedObjects.append(lookupImportController)

        # ----------------
        # Create the update branch controller
        branchUpdateController = BranchUpdateController(
            liveDbWriteApi=liveDbApi.writeApi,
            tupleObservable=tupleObservable,
            liveEditController=branchLiveEditController,
            dbSessionCreator=self.dbSessionCreator,
        )
        self._loadedObjects.append(branchUpdateController)

        # ----------------
        # Create the Watch Grid Controller
        liveDbWatchController = LiveDbWatchController(
            liveDbWriteApi=liveDbApi.writeApi,
            liveDbReadApi=liveDbApi.readApi,
            dbSessionCreator=self.dbSessionCreator,
        )
        self._loadedObjects.append(liveDbWatchController)

        # ----------------
        # Create the GRID API for the client
        self._loadedObjects.extend(
            ClientGridLoaderRpc(
                liveDbWatchController=liveDbWatchController,
                dbSessionCreator=self.dbSessionCreator,
            ).makeHandlers()
        )

        # ----------------
        # Create the Branch Index for the client
        self._loadedObjects.extend(
            BranchIndexChunkLoadRpc(
                dbSessionCreator=self.dbSessionCreator
            ).makeHandlers()
        )

        # ----------------
        # Create the LOCATION API for the client
        self._loadedObjects.extend(
            ClientLocationIndexLoaderRpc(
                dbSessionCreator=self.dbSessionCreator
            ).makeHandlers()
        )

        # ----------------
        # Initialise the API object that will be shared with other plugins
        self._api = DiagramApi(
            statusController,
            dispImportController,
            lookupImportController,
            branchUpdateController,
            self.dbSessionCreator,
        )
        self._loadedObjects.append(self._api)

        # ----------------
        # Create the Action Processor
        self._loadedObjects.append(
            makeTupleActionProcessorHandler(
                statusController,
                branchUpdateController,
                branchLiveEditController,
            )
        )

        # ----------------
        # Start the queue controller

        settings = yield self._loadSettings()

        if settings[DISP_COMPILER_ENABLED]:
            dispCompilerQueueController.start()

        if settings[GRID_COMPILER_ENABLED]:
            gridKeyCompilerQueueController.start()

        if settings[BRANCH_COMPILER_ENABLED]:
            branchIndexCompilerQueueController.start()

        if settings[LOCATION_COMPILER_ENABLED]:
            locationIndexCompilerQueueController.start()

        logger.debug("Started")

    def stop(self):
        """Stop

        This method is called by the platform to tell the peek app to shutdown and stop
        everything it's doing
        """
        # Shutdown and dereference all objects we constructed when we started
        while self._loadedObjects:
            self._loadedObjects.pop().shutdown()

        self._api = None

        logger.debug("Stopped")

    def unload(self):
        """Unload

        This method is called after stop is called, to unload any last resources
        before the PLUGIN is unlinked from the platform

        """
        logger.debug("Unloaded")

    @property
    def publishedServerApi(self) -> object:
        """Published Server API

        :return  class that implements the API that can be used by other Plugins on this
        platform service.
        """
        return self._api

    @deferToThreadWrapWithLogger(logger)
    def _loadSettings(self):
        dbSession = self.dbSessionCreator()
        try:
            return {
                globalProperties[p.key]: p.value
                for p in globalSetting(dbSession).propertyObjects
            }

        finally:
            dbSession.close()

    ###### Implement PluginServerWorkerEntryHookABC



"""use darkcolor lightcolor

Peek Plugin Database Migration Script

Revision ID: 0ad02369aaea
Revises: 0db3aedfee95
Create Date: 2022-10-20 15:56:02.384820

"""
from sqlalchemy.orm import Session

from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram.tuples.ColorUtil import invertColor

# revision identifiers, used by Alembic.
revision = "0ad02369aaea"
down_revision = "0db3aedfee95"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column(
        "DispColor",
        "color",
        schema="pl_diagram",
        nullable=True,
        new_column_name="darkColor",
    )

    op.add_column(
        "DispColor",
        sa.Column("lightColor", sa.String(), nullable=True),
        schema="pl_diagram",
    )
    op.execute(
        """
        UPDATE pl_diagram."DispColor"
        SET "darkColor" = null
        WHERE "darkColor" = 'None'
        """
    )

    session = Session(bind=op.get_bind())
    rows = (
        session.query(DispColor).filter(DispColor.darkColor.isnot(None)).all()
    )

    for row in rows:
        row.lightColor = invertColor(row.darkColor, "#fff")
    session.commit()


def downgrade():
    op.drop_column("DispColor", "lightColor", schema="pl_diagram")
    op.alter_column(
        "DispColor",
        "darkColor",
        nullable=True,
        new_column_name="color",
        schema="pl_diagram",
    )



from txhttputil.util.ModuleUtil import filterModules


def loadPublicTuples():
    """Load Public Tuples

    In this method, we load the public tuples.
    This registers them so the Vortex can reconstructed them from
    serialised data.

    """

    for mod in filterModules(__name__, __file__):
        __import__(mod, locals(), globals())

    from .branches import loadPublicBranchTuples

    loadPublicBranchTuples()

    from .lookups import loadLookupTuples

    loadLookupTuples()

    from .shapes import loadShapeTuples

    loadShapeTuples()

    from .model import loadModelTuples

    loadModelTuples()

    from .grids import loadGridTuples

    loadGridTuples()



from copy import copy
from typing import List

from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler
from vortex.handler.TupleDataObserverClient import TupleDataObserverClient

from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram._private.storage.Lookups import DispLayer
from peek_plugin_diagram._private.storage.Lookups import DispLevel
from peek_plugin_diagram._private.storage.Lookups import DispLineStyle
from peek_plugin_diagram._private.storage.Lookups import DispTextStyle

lookupCachePayloadFilt = dict(key="client.lookup.update")
lookupCachePayloadFilt.update(diagramFilt)


class LookupCacheController:
    """Lookup Cache Controller

    This class caches the lookups in each client.

    """

    #: This stores the cache of grid data for the clients
    _levelLookups: List[DispLevel] = None
    _layerLookups: List[DispLayer] = None
    _colorLookups: List[DispColor] = None
    _lineStyleLookups: List[DispLineStyle] = None
    _textStyleLookups: List[DispTextStyle] = None

    def __init__(self, tupleObserver: TupleDataObserverClient):
        self._tupleObserver = tupleObserver
        self._tupleObservable = None
        self._vortexMsgCache: dict[str, bytes] = {}

    def setTupleObservable(self, tupleObservable: TupleDataObservableHandler):
        self._tupleObservable = tupleObservable

    def start(self):
        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(DispLevel.tupleType(), {})
            ).subscribe(self._processNewTuples)
        )

        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(DispLayer.tupleType(), {})
            ).subscribe(self._processNewTuples)
        )

        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(DispColor.tupleType(), {})
            ).subscribe(self._processNewTuples)
        )

        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(DispLineStyle.tupleType(), {})
            ).subscribe(self._processNewTuples)
        )

        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(DispTextStyle.tupleType(), {})
            ).subscribe(self._processNewTuples)
        )

    def shutdown(self):
        self._tupleObservable = None
        self._tupleObserver = None

        self._vortexMsgCache = {}

        self._levelLookups = []
        self._layerLookups = []
        self._colorLookups = []
        self._lineStyleLookups = []
        self._textStyleLookups = []

    def _processNewTuples(self, lookupTuples):
        if not lookupTuples:
            return

        firstTupleType = lookupTuples[0].tupleType()
        if DispLevel.tupleType() == firstTupleType:
            self._levelLookups = lookupTuples

        elif DispLayer.tupleType() == firstTupleType:
            self._layerLookups = lookupTuples

        elif DispColor.tupleType() == firstTupleType:
            self._colorLookups = lookupTuples

        elif DispLineStyle.tupleType() == firstTupleType:
            self._lineStyleLookups = lookupTuples

        elif DispTextStyle.tupleType() == firstTupleType:
            self._textStyleLookups = lookupTuples

        else:
            raise NotImplementedError(
                "Cache not implemented for %s" % firstTupleType
            )

        self._vortexMsgCache.pop(firstTupleType, None)

        self._tupleObservable.notifyOfTupleUpdate(
            TupleSelector(firstTupleType, {})
        )

    def lookups(self, lookupTupleType) -> List:
        if DispLevel.tupleType() == lookupTupleType:
            return copy(self._levelLookups)

        if DispLayer.tupleType() == lookupTupleType:
            return copy(self._layerLookups)

        if DispColor.tupleType() == lookupTupleType:
            return copy(self._colorLookups)

        if DispLineStyle.tupleType() == lookupTupleType:
            return copy(self._lineStyleLookups)

        if DispTextStyle.tupleType() == lookupTupleType:
            return copy(self._textStyleLookups)

        raise NotImplementedError(
            "Cache not implemented for %s" % lookupTupleType
        )

    def cachedVortexMsgBlocking(
        self, lookupTupleType: str, filt: dict
    ) -> bytes:
        if lookupTupleType in self._vortexMsgCache:
            return self._vortexMsgCache[lookupTupleType]

        data = self.lookups(lookupTupleType=lookupTupleType)

        # Create the vortex message
        vortexMsg = (
            Payload(filt, tuples=data).makePayloadEnvelope().toVortexMsg()
        )
        self._vortexMsgCache[lookupTupleType] = vortexMsg
        return vortexMsg



from txhttputil.util.ModuleUtil import filterModules


def loadPrivateTuples():
    """Load Private Tuples

    In this method, we load the private tuples.
    This registers them so the Vortex can reconstructed them from
    serialised data.

    """
    for mod in filterModules(__name__, __file__):
        __import__(mod, locals(), globals())

    from peek_plugin_diagram._private.tuples import grid

    for mod in filterModules(grid.__name__, grid.__file__):
        __import__(mod, locals(), globals())

    from peek_plugin_diagram._private.tuples import branch

    for mod in filterModules(branch.__name__, branch.__file__):
        __import__(mod, locals(), globals())

    from peek_plugin_diagram._private.tuples import location_index

    for mod in filterModules(location_index.__name__, location_index.__file__):
        __import__(mod, locals(), globals())



from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple

from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)


@addTupleType
class EncodedLocationIndexTuple(Tuple, ACIEncodedChunkTupleABC):
    __tupleType__ = diagramTuplePrefix + "EncodedLocationIndexTuple"

    modelSetKey: str = TupleField()
    indexBucket: str = TupleField()

    # The LocationIndexTuple pre-encoded to a Payload
    encodedLocationIndexTuple: bytes = TupleField()
    lastUpdate: str = TupleField()

    @property
    def ckiChunkKey(self):
        return self.indexBucket

    @property
    def ckiEncodedData(self):
        return self.encodedLocationIndexTuple

    @property
    def ckiHasEncodedData(self) -> bool:
        return bool(self.encodedLocationIndexTuple)

    @property
    def ckiLastUpdate(self):
        return self.lastUpdate



from typing import List

from vortex.Tuple import Tuple

from peek_plugin_diagram._private.worker.api.WorkerDiagramLookupApiImpl import (
    WorkerDiagramLookupApiImpl,
)


class WorkerDiagramLookupApi:
    @classmethod
    def getColors(cls) -> List[Tuple]:
        return WorkerDiagramLookupApiImpl.getColors()

    @classmethod
    def getLineStyles(cls) -> List[Tuple]:
        return WorkerDiagramLookupApiImpl.getLineStyles()

    @classmethod
    def getTextStyles(cls) -> List[Tuple]:
        return WorkerDiagramLookupApiImpl.getTextStyles()

    @classmethod
    def getLayers(cls) -> List[Tuple]:
        return WorkerDiagramLookupApiImpl.getLayers()

    @classmethod
    def getLevels(cls) -> List[Tuple]:
        return WorkerDiagramLookupApiImpl.getLevels()



from typing import List
from typing import Union

from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram._private.storage.Lookups import DispLineStyle
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase


class ShapePoly(ShapeBase):
    @staticmethod
    def lineColor(disp) -> DispColor:
        return disp["lcl"]

    @staticmethod
    def lineStyle(disp) -> DispLineStyle:
        return disp["lsl"]

    @staticmethod
    def lineWidth(disp) -> float:
        return disp["w"]

    @staticmethod
    def geom(disp) -> List[float]:
        return disp["g"]

    @staticmethod
    def pointCount(disp) -> int:
        return int(len(ShapePoly.geom(disp)) / 2)

    @staticmethod
    def lastPoint(disp) -> Union[Point, None]:
        pointCount = len(ShapePoly.geom(disp))
        if pointCount == 0:
            return None

        return Point(x=disp["g"][pointCount - 1], y=disp["g"][pointCount - 2])



import json
from collections import namedtuple
from typing import Union

from peek_plugin_diagram._private.storage.Lookups import DispLayer
from peek_plugin_diagram._private.storage.Lookups import DispLevel

Point = namedtuple("Point", ["x", "y"])


class ShapeType:
    ellipse = 0
    polygon = 1
    polyline = 2
    text = 3
    group = 4
    groupPointer = 5
    edgeTemplate = 6
    null_ = 7
    curvedText = 8


class ShapeActionEnum:
    none = 0  # Or null
    positionOn = 1


class ShapeBase:
    TYPE_DT = "DT"
    TYPE_DCT = "DCT"
    TYPE_DPG = "DPG"
    TYPE_DPL = "DPL"
    TYPE_DE = "DE"
    TYPE_DG = "DG"
    TYPE_DGP = "DGP"
    TYPE_DET = "DET"
    TYPE_DN = "DN"

    DEEP_COPY_FIELDS_TO_IGNORE = ["bounds", "disps", "dispGroup"]

    _typeMapInit = False
    _typeMap = {}

    # Lazy instantiation, because the string types are defined elsewhere
    @classmethod
    @property
    def typeMap(cls) -> dict:
        if not cls._typeMapInit:
            cls._typeMapInit = True
            cls._typeMap[cls.TYPE_DT] = [ShapeType.text, "Text"]
            cls._typeMap[cls.TYPE_DCT] = [ShapeType.curvedText, "CurvedText"]
            cls._typeMap[cls.TYPE_DPG] = [ShapeType.polygon, "Polygon"]
            cls._typeMap[cls.TYPE_DPL] = [ShapeType.polyline, "Polyline"]
            cls._typeMap[cls.TYPE_DE] = [ShapeType.ellipse, "Ellipse"]
            cls._typeMap[cls.TYPE_DG] = [ShapeType.group, "Group"]
            cls._typeMap[cls.TYPE_DGP] = [
                ShapeType.groupPointer,
                "GroupPointer",
            ]
            cls._typeMap[cls.TYPE_DET] = [
                ShapeType.edgeTemplate,
                "EdgeTemplate",
            ]
            cls._typeMap[cls.TYPE_DN] = [ShapeType.null_, "Deleted Shape"]

        return cls._typeMap

    # Helper query methods
    @staticmethod
    def typeOf(disp) -> ShapeType:
        return (ShapeBase.typeMap)[disp["_tt"]][0]

    @staticmethod
    def hasColor(disp) -> bool:
        if disp["lcl"]:
            return True
        if disp["fcl"]:
            return True
        if disp["cl"]:
            return True
        return False

    @staticmethod
    def niceName(disp) -> str:
        return (ShapeBase.typeMap)[disp["_tt"]][1]

    # getters
    @staticmethod
    def type(disp) -> str:
        return disp.get("_tt")

    @staticmethod
    def id(disp) -> int:
        return disp.get("id")

    @staticmethod
    def zOrder(disp) -> int:
        if disp["z"]:
            return disp.get("z")
        return 0

    @staticmethod
    def hashId(disp) -> str:
        return disp.get("hid")

    @staticmethod
    def replacesHashId(disp) -> str:
        return disp.get("rid")

    @staticmethod
    def groupId(disp) -> Union[int, None]:
        return disp.get("gi")

    @staticmethod
    def branchId(disp) -> int:
        return disp.get("bi")

    @staticmethod
    def branchStage(disp) -> int:
        return disp.get("bs")

    @staticmethod
    def level(disp) -> DispLevel:
        return disp.get("lel", {})

    @staticmethod
    def layer(disp) -> DispLayer:
        return disp.get("lal", {})

    @staticmethod
    def isOverlay(disp) -> bool:
        return disp.get("o")

    @staticmethod
    def isSelectable(disp) -> bool:
        return disp.get("s")

    @staticmethod
    def key(disp) -> Union[str, None]:
        return disp.get("k")

    @staticmethod
    def action(disp) -> Union[ShapeActionEnum, None]:
        # ShapeActionEnum
        return disp.get("a")

    @staticmethod
    def data(disp) -> dict:
        if disp["d"] is None:
            return {}
        return json.loads(disp["d"])



"""removed more string sizes

Peek Plugin Database Migration Script

Revision ID: 473d5f6a97e8
Revises: 6f545d4166dc
Create Date: 2019-06-10 22:21:54.762566

"""

# revision identifiers, used by Alembic.
revision = "473d5f6a97e8"
down_revision = "6f545d4166dc"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.alter_column(
        "DispBase", "importHash", type_=sa.String(), nullable=True, schema="pl_diagram"
    )
    op.alter_column(
        "DispBase",
        "importGroupHash",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )

    op.alter_column(
        "LiveDbDispLink",
        "dispAttrName",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "LiveDbDispLink",
        "liveDbKey",
        type_=sa.String(),
        nullable=False,
        schema="pl_diagram",
    )

    op.alter_column(
        "LiveDbDispLink",
        "importKeyHash",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "LiveDbDispLink",
        "importGroupHash",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "LiveDbDispLink",
        "importDispHash",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "LiveDbDispLink",
        "propsJson",
        type_=sa.String(),
        nullable=True,
        schema="pl_diagram",
    )


def downgrade():
    pass



import logging

from vortex.Tuple import Tuple
from vortex.rpc.RPC import vortexRPC

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkLoadRpcABC import (
    ACIChunkLoadRpcABC,
)
from peek_plugin_base.PeekVortexUtil import peekBackendNames
from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.storage.branch.BranchIndexEncodedChunk import (
    BranchIndexEncodedChunk,
)
from peek_plugin_diagram._private.tuples.branch.BranchIndexUpdateDateTuple import (
    BranchIndexUpdateDateTuple,
)

logger = logging.getLogger(__name__)


class BranchIndexChunkLoadRpc(ACIChunkLoadRpcABC):
    def makeHandlers(self):
        """Make Handlers

        In this method we start all the RPC handlers
        start() returns an instance of it's self so we can simply yield the result
        of the start method.

        """

        yield self.loadBranchIndexChunks.start(funcSelf=self)
        yield self.loadBranchIndexDelta.start(funcSelf=self)
        logger.debug("RPCs started")

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        timeoutSeconds=120,
        additionalFilt=diagramFilt,
        deferToThread=True,
    )
    def loadBranchIndexDelta(self, indexEncodedPayload: bytes) -> bytes:
        return self.ckiChunkIndexDeltaBlocking(
            indexEncodedPayload,
            BranchIndexEncodedChunk,
            BranchIndexUpdateDateTuple,
        )

    # -------------
    @vortexRPC(
        peekServerName,
        acceptOnlyFromVortex=peekBackendNames,
        timeoutSeconds=120,
        additionalFilt=diagramFilt,
        deferToThread=True,
    )
    def loadBranchIndexChunks(self, chunkKeys: list[str]) -> list[Tuple]:
        """Update Page Loader Status

        Tell the server of the latest status of the loader

        """
        return self.ckiInitialLoadChunksPayloadBlocking(
            chunkKeys, BranchIndexEncodedChunk
        )



from typing import Annotated
from typing import Dict
from typing import List
from typing import Optional

from vortex.Tuple import Tuple

from peek_plugin_diagram.tuples.grids.GridKeyTuple import GridKeyTuple


class WorkerDiagramGridApi:
    @classmethod
    def getGridKeys(
        cls,
        modelSetKey: str,
        coordSetKey: str,
        boundingBox: Annotated[List[float], 4] = None,
    ):
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.getGridKeys(
            modelSetKey=modelSetKey,
            coordSetKey=coordSetKey,
            boundingBox=boundingBox,
        )

    @classmethod
    def getShapesByGridKeys(cls, gridKeys) -> Dict[str, Tuple]:
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.getShapesByGridKeys(gridKeys=gridKeys)

    @classmethod
    def linkShapes(cls, decodedCompiledGridTuplesByGridKey: Dict):
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.linkShapes(
            decodedCompiledGridTuplesByGridKey
        )

    @classmethod
    def compileShapes(
        cls,
        decodedCompiledGridTuples: List,
        levelsOrderedByOrder,
        layersOrderedByOrder,
        shapeSelectionZoomLevel=2.0,
        enabledLayerKeys: Optional[list[str]] = None,
        enabledBranchKeys: Optional[list[str]] = None,
    ):
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.compileShapes(
            decodedCompiledGridTuples=decodedCompiledGridTuples,
            levelsOrderedByOrder=levelsOrderedByOrder,
            layersOrderedByOrder=layersOrderedByOrder,
            shapeSelectionZoomLevel=shapeSelectionZoomLevel,
            enabledLayerKeys=enabledLayerKeys,
            enabledBranchKeys=enabledBranchKeys,
        )

    @classmethod
    def getGridKeyTuplesFromShapeKeys(
        cls,
        modelSetKey,
        coordSetKey,
        shapeKeys: list[str],
        smallestGridKeySize: Optional[bool] = False,
    ) -> List[GridKeyTuple]:
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.getGridKeyTuplesFromShapeKeys(
            modelSetKey=modelSetKey,
            coordSetKey=coordSetKey,
            shapeKeys=shapeKeys,
            smallestGridKeySize=smallestGridKeySize,
        )

    @classmethod
    def filterShapesByShapeKey(
        cls, shapes: list[dict], filterForKeys: set[str]
    ) -> list[dict]:
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        return WorkerDiagramGridApiImpl.filterShapesByShapeKey(
            shapes=shapes, filterForKeys=filterForKeys
        )



import logging

from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet, ModelSet
from peek_plugin_diagram.server.DiagramViewerApiABC import DiagramViewerApiABC
from peek_plugin_diagram.tuples.model.DiagramCoordSetTuple import DiagramCoordSetTuple
from vortex.DeferUtil import deferToThreadWrapWithLogger

logger = logging.getLogger(__name__)


class DiagramViewerApi(DiagramViewerApiABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    def shutdown(self):
        pass

    @deferToThreadWrapWithLogger(logger)
    def getCoordSets(self, modelSetKey: str):
        ormSession = self._ormSessionCreator()
        try:
            all = (
                ormSession.query(ModelCoordSet)
                .join(ModelSet)
                .filter(ModelSet.name == modelSetKey)
                .all()
            )

            coordSetTuples = []
            for obj in all:
                coordSetTuples.append(DiagramCoordSetTuple(key=obj.key, name=obj.name))

            return coordSetTuples

        finally:
            ormSession.close()



import logging

from peek_plugin_base.worker.PluginWorkerEntryHookABC import (
    PluginWorkerEntryHookABC,
)
from peek_plugin_diagram._private.worker.tasks import (
    GridCompilerTask,
    ImportDispTask,
    DispCompilerTask,
    LocationIndexCompilerTask,
)
from peek_plugin_diagram._private.worker.tasks.branch import (
    BranchIndexCompilerTask,
    BranchIndexImporterTask,
    BranchIndexUpdaterTask,
)
from peek_plugin_diagram.tuples import loadPublicTuples

logger = logging.getLogger(__name__)


class WorkerEntryHook(PluginWorkerEntryHookABC):
    def load(self):
        loadPublicTuples()
        logger.debug("loaded")

    def start(self):
        logger.debug("started")

    def stop(self):
        logger.debug("stopped")

    def unload(self):
        logger.debug("unloaded")

    @property
    def celeryAppIncludes(self):
        return [
            BranchIndexUpdaterTask.__name__,
            BranchIndexCompilerTask.__name__,
            BranchIndexImporterTask.__name__,
            DispCompilerTask.__name__,
            GridCompilerTask.__name__,
            ImportDispTask.__name__,
            LocationIndexCompilerTask.__name__,
        ]



from typing import Dict
from typing import List
from typing import Union

from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase


class ShapeGroupPoint(ShapeBase):
    @staticmethod
    def targetGroupId(disp) -> int:
        return disp["tg"]

    @staticmethod
    def targetGroupCoordSetId(disp) -> Union[int, None]:
        if disp["tn"] is None or "|" not in disp["tn"]:
            return None

        number, _, _ = disp["tn"].partition("|")
        return int(number)

    @staticmethod
    def targetGroupName(disp) -> Union[str, None]:
        if disp["tn"] is None or "|" not in disp["tn"]:
            return None

        _, _, groupName = disp["tn"].partition("|")
        return groupName

    @staticmethod
    def verticalScale(disp) -> float:
        return disp["vs"]

    @staticmethod
    def horizontalScale(disp) -> float:
        return disp["hs"]

    @staticmethod
    def rotation(disp) -> float:
        if disp["r"] is None:
            return 0
        return disp["r"]

    @staticmethod
    def center(disp) -> Point:
        return Point(x=disp["g"][0], y=disp["g"][1])

    @staticmethod
    def geom(disp) -> List[float]:
        return disp["g"]

    @staticmethod
    def disps(disp) -> List[Dict]:
        return disp["disps"]



from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ShapeLevelTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "ShapeLevelTuple"

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()

    name: str = TupleField()
    order: int = TupleField()
    minZoom: float = TupleField()
    maxZoom: float = TupleField()

    modelSetKey: str = TupleField()
    coordSetKey: str = TupleField()

    importHash: str = TupleField()

    showForEdit: bool = TupleField()

    blockApiUpdate: bool = TupleField()

    def isVisibleAtZoom(self, zoom: float) -> bool:
        return self.minZoom <= zoom < self.maxZoom



from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ShapeLayerTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "ShapeLayerTuple"

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()

    name: str = TupleField()
    order: int = TupleField()
    selectable: bool = TupleField()
    visible: bool = TupleField()

    modelSetKey: str = TupleField()

    importHash: str = TupleField()

    showForEdit: bool = TupleField()

    blockApiUpdate: bool = TupleField()

    def isVisibleAtZoom(self, zoom: float) -> bool:
        return self.minZoom <= zoom < self.maxZoom



import logging
from typing import Union

from sqlalchemy.orm import joinedload
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.Tuple import TUPLE_TYPES_BY_NAME
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.storage.Lookups import DispLevel
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet

logger = logging.getLogger(__name__)


class ServerLookupTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        session = self._ormSessionCreator()
        try:
            Lookup = TUPLE_TYPES_BY_NAME[tupleSelector.name]

            if Lookup == DispLevel:
                all = (
                    session.query(DispLevel)
                    .options(
                        joinedload(DispLevel.coordSet).joinedload(
                            ModelCoordSet.modelSet
                        )
                    )
                    .all()
                )

            else:
                all = (
                    session.query(Lookup)
                    .options(joinedload(Lookup.modelSet))
                    .all()
                )

            for item in all:
                item.setTupleFields()

            return Payload(filt, tuples=all).makePayloadEnvelope().toVortexMsg()

        finally:
            session.close()



"""updated float precision in zoom levels

Peek Plugin Database Migration Script

Revision ID: a66e033b8227
Revises: 5307eb21c742
Create Date: 2022-07-04 10:21:09.213318

"""

# revision identifiers, used by Alembic.
revision = "a66e033b8227"
down_revision = "5307eb21c742"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "DispLevel",
        "minZoom",
        type_=sa.Float(precision="5"),
        nullable=True,
        schema="pl_diagram",
    )
    op.alter_column(
        "DispLevel",
        "maxZoom",
        type_=sa.Float(precision="5"),
        nullable=True,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("DispLevel", "minZoom", type_=sa.Float(), nullable=True)
    op.alter_column("DispLevel", "maxZoom", type_=sa.Float(), nullable=True)
    # ### end Alembic commands ###



"""added spacingBetweenTexts in textstyle

Peek Plugin Database Migration Script

Revision ID: 5307eb21c742
Revises: a4c59bcaaf88
Create Date: 2022-06-14 15:20:36.877084

"""

# revision identifiers, used by Alembic.
revision = "5307eb21c742"
down_revision = "a4c59bcaaf88"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispTextStyle",
        sa.Column(
            "spacingBetweenTexts",
            sa.Float(),
            nullable=True,
        ),
        schema="pl_diagram",
    )
    op.execute(
        """UPDATE
            "pl_diagram"."DispTextStyle"
            SET
            "spacingBetweenTexts" = 100;"""
    )
    op.alter_column(
        "DispTextStyle",
        "spacingBetweenTexts",
        nullable=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DispTextStyle", "spacingBetweenTexts", schema="pl_diagram")
    # ### end Alembic commands ###



"""add DispBase.key index

Peek Plugin Database Migration Script

Revision ID: 57e88ad9d5cb
Revises: 0ad02369aaea
Create Date: 2023-02-20 14:07:57.536490

"""

# revision identifiers, used by Alembic.
revision = "57e88ad9d5cb"
down_revision = "0ad02369aaea"
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    op.create_index(
        "idx_Disp_key", "DispBase", ["key"], unique=False, schema="pl_diagram"
    )


def downgrade():
    op.drop_index("idx_Disp_key", table_name="DispBase", schema="pl_diagram")



from typing import Optional

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportDispTextStyleTuple(Tuple):
    """Import Display Text Style Tuple"""

    __tupleType__ = diagramTuplePrefix + "ImportDispTextStyleTuple"

    name: str = TupleField()

    fontName: str = TupleField()
    fontSize: float = TupleField()

    STYLE_BOLD = "bold"
    fontStyle: str = TupleField()

    scalable: bool = TupleField(defaultValue=True)
    scaleFactor: float = TupleField(defaultValue=1)

    importHash: str = TupleField()

    modelSetKey: str = TupleField()

    #: a parameter to put white space between text repeats along a
    # path in 'ImportDispCurvedTextTuple.geom'.
    #  'spacingBetweenTexts' is in range of (0, +∞)
    #  The spacing is the spacing distance at zoom level 1, in the coordinate
    #  system unit.
    spacingBetweenTexts: float = TupleField(defaultValue=100)

    borderWidth: Optional[float] = TupleField()

    showForEdit: bool = TupleField(defaultValue=False)

    blockApiUpdate: bool = TupleField(defaultValue=False)

    wrapTextAtChars: int = TupleField(defaultValue=None)



from typing import Union

from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram._private.storage.Lookups import DispTextStyle
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeText import ShapeText
from peek_plugin_diagram.worker.canvas_shapes.ShapeText import (
    TextHorizontalAlign,
)
from peek_plugin_diagram.worker.canvas_shapes.ShapeText import TextVerticalAlign


class ShapeCurvedText(ShapeBase):
    @staticmethod
    def textStyle(disp) -> DispTextStyle:
        return disp.get("fsl", DispTextStyle())

    @staticmethod
    def borderColor(disp) -> DispColor:
        return disp.get("bcl", DispColor())

    @staticmethod
    def color(disp) -> DispColor:
        return disp.get("cl", DispColor())

    @staticmethod
    def verticalAlign(disp) -> int:
        val = disp.get("va")

        if val == TextVerticalAlign.top:
            return TextVerticalAlign.top

        if val == TextVerticalAlign.bottom:
            return TextVerticalAlign.bottom

        return TextVerticalAlign.center

    @staticmethod
    def horizontalAlign(disp) -> int:
        val = disp.get("ha")

        if val == TextHorizontalAlign.left:
            return TextHorizontalAlign.left

        if val == TextHorizontalAlign.right:
            return TextHorizontalAlign.right

        return TextHorizontalAlign.center

    @staticmethod
    def rotation(disp) -> int:
        return disp.get("r")

    @staticmethod
    def text(disp) -> str:
        return disp.get("te", "")

    @staticmethod
    def height(disp) -> Union[int, None]:
        return disp.get("th", None)

    @staticmethod
    def horizontalStretch(disp) -> float:
        return disp.get("hs")

    @staticmethod
    def centerPointX(disp) -> float:
        return disp["g"][0]

    @staticmethod
    def centerPointY(disp) -> float:
        return disp["g"][1]

    @staticmethod
    def center(disp) -> Point:
        return Point(
            x=ShapeText.centerPointX(disp), y=ShapeText.centerPointY(disp)
        )

    @staticmethod
    def spacingBetweenTexts(disp) -> float:
        return disp.get("sbt")

    @staticmethod
    def geom(disp):
        return disp.get("g")



"""enabled nulls for text

Peek Plugin Database Migration Script

Revision ID: 4925c894757b
Revises: e9d80c4c087d
Create Date: 2017-11-04 14:43:04.703237

"""

# revision identifiers, used by Alembic.
revision = "4925c894757b"
down_revision = "e9d80c4c087d"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.alter_column(
        "DispText", "text", type_=sa.String(), nullable=True, schema="pl_diagram"
    )


def downgrade():
    op.alter_column(
        "DispText",
        "text",
        type_=sa.String(length=1000),
        nullable=False,
        schema="pl_diagram",
    )



import logging
from typing import Optional
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.GridCacheController import (
    GridCacheController,
)
from peek_plugin_diagram._private.storage.ModelSet import makeDispGroupGridKey
from peek_plugin_diagram._private.tuples.GroupDispsTuple import GroupDispsTuple
from peek_plugin_diagram._private.tuples.grid.EncodedGridTuple import (
    EncodedGridTuple,
)

logger = logging.getLogger(__name__)


class ClientGroupDispsTupleProvider(TuplesProviderABC):
    def __init__(self, gridCacheController: GridCacheController):
        self.gridCacheController = gridCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        coordSetId = tupleSelector.selector["coordSetId"]
        gridKey = makeDispGroupGridKey(coordSetId)
        grid: Optional[
            EncodedGridTuple
        ] = self.gridCacheController.encodedChunk(gridKey)

        groupDispTuple = GroupDispsTuple()
        groupDispTuple.coordSetId = coordSetId
        groupDispTuple.encodedGridTuple = (
            grid.encodedGridTuple if grid else None
        )

        payloadEnvelope = yield Payload(
            filt, tuples=[groupDispTuple]
        ).makePayloadEnvelopeDefer()

        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



import logging
import sys
from unittest import TestCase


class WorkerDiagramGridApiImpl(TestCase):
    def setUp(self):
        # setup for `PeekPlatformConfig.config.bashLocation`
        logging.basicConfig(
            stream=sys.stdout,
            format="%(asctime)s %(levelname)s %(name)s:%(message)s",
            datefmt="%d-%b-%Y %H:%M:%S",
            level=logging.DEBUG,
        )
        from peek_platform import PeekPlatformConfig

        from peek_plugin_base.PeekVortexUtil import peekWorkerName

        PeekPlatformConfig.componentName = peekWorkerName

        # The config depends on the componentName, order is important
        from peek_worker_service.PeekWorkerConfig import PeekWorkerConfig

        PeekPlatformConfig.config = PeekWorkerConfig()

        # setup for `CeleryDbConn`
        from peek_plugin_base.worker import CeleryDbConn

        CeleryDbConn._dbConnectString = (
            PeekPlatformConfig.config.dbConnectString
        )
        CeleryDbConn._dbEngineArgs = PeekPlatformConfig.config.dbEngineArgs

    # Disabled, this needs specific data
    def _test_getGridKeysFromShapeKeys_All(self):
        from .WorkerDiagramGridApiImpl import WorkerDiagramGridApiImpl

        result = WorkerDiagramGridApiImpl.getGridKeyTuplesFromShapeKeys(
            "pofDiagram", "Distribution World", testKeys
        )
        self.assertTrue(result)
        print(result)

    # Disabled, this needs specific data
    def _test_getGridKeysFromShapeKeys_Smallest(self):
        from .WorkerDiagramGridApiImpl import WorkerDiagramGridApiImpl

        result = WorkerDiagramGridApiImpl.getGridKeyTuplesFromShapeKeys(
            "pofDiagram", "Distribution World", testKeys
        )
        self.assertTrue(result)
        print(result)

    # Disabled, this needs specific data
    def _test_filterShapesByShapeKey(self):
        from peek_plugin_diagram._private.worker.api.WorkerDiagramGridApiImpl import (
            WorkerDiagramGridApiImpl,
        )

        WorkerDiagramGridApiImpl._DEBUG_LOGGING = True

        gridKeyTuples = WorkerDiagramGridApiImpl.getGridKeys(
            "pofDiagram", "Distribution World"
        )[:50]
        self.assertTrue(gridKeyTuples)

        decodedCompiledGridTuples = (
            WorkerDiagramGridApiImpl.getShapesByGridKeys(
                [g.gridKey for g in gridKeyTuples]
            )
        )
        self.assertTrue(decodedCompiledGridTuples)

        traceKeys = set(testKeys)
        for decodedCompiledGridTuple in decodedCompiledGridTuples.values():
            print(f"running for {decodedCompiledGridTuple.gridKey}")
            print(
                str(
                    [
                        s.get("k")
                        for s in decodedCompiledGridTuple.shapes
                        if s.get("k") and s.get("g")
                    ]
                )
            )
            decodedCompiledGridTuple.shapes = (
                WorkerDiagramGridApiImpl.filterShapesByShapeKey(
                    decodedCompiledGridTuple.shapes, traceKeys
                )
            )


testKeys = [
    "k00020aa6COMP",
    "k00020b68COMP",
    "k0001a7cbCOMP",
    "k0001a846COMP",
    "k0001a87eCOMP",
    "k0001a705COMP",
    "k0001a902COMP",
    "k0001a848COMP",
    "k0001a87fCOMP",
    "k0001a8f8COMP",
    "k0001a901COMP",
    "k0001a8a1COMP",
    "k00020b2cCOMP",
    "k00020b15COMP",
    "k00020a7dCOMP",
    "k00020b22COMP",
    "k0001a82bCOMP",
    "k00020b7aCOMP",
    "k00020acbCOMP",
    "k00020b74COMP",
    "k00020af0COMP",
    "k00020afeCOMP",
    "k00020b96COMP",
    "k00020b3aCOMP",
    "k00020b57COMP",
    "k0001a864COMP",
    "k0001a582COMP",
    "k0007db66HOT",
    "k0001a429COMP",
    "k0007daedHOT",
    "k0001a43bCOMP",
    "k0007daf6HOT",
    "k000207a6COMP",
    "e0001b43eHOT",
    "k000207a5COMP",
    "e0001b43dHOT",
    "k000207a8COMP",
    "e00018fc1HOT",
    "k000207a9COMP",
    "e00018fc2HOT",
    "k000207acCOMP",
    "k0002070aCOMP",
    "e0001263eHOT",
    "k000206f4COMP",
    "e0001263fHOT",
    "k000206f8COMP",
    "e00012640HOT",
    "k0006c0beCOMP",
    "k000a1881COMP",
    "k0006bf89HOT",
    "k000a18bbCOMP",
    "k0006bf8aHOT",
    "k000a1a17COMP",
    "k0006bf8bHOT",
    "k000a12e7COMP",
    "k00069a3bHOT",
    "k000a19ccCOMP",
    "k00069d55HOT",
    "k000a1607COMP",
    "k00069bb4HOT",
    "k000a175cCOMP",
    "k00069c29HOT",
    "k000a19f3COMP",
    "k00069d67HOT",
    "k000a16baCOMP",
    "k00069bf7HOT",
    "k000a1333COMP",
    "k00069a5cHOT",
    "k000a1817COMP",
    "k00069c84HOT",
    "k000a1463COMP",
    "k00069aeeHOT",
    "k000a157bCOMP",
    "k00069b76HOT",
    "k000a19ceCOMP",
    "k00069d56HOT",
    "k000a18beCOMP",
    "k00069cdcHOT",
    "k000a1577COMP",
    "k00069b74HOT",
    "k000a1994COMP",
    "k00069d34HOT",
    "k000a15f8COMP",
    "k00069bacHOT",
    "k000a15b4COMP",
    "k00069b90HOT",
    "k000a12baCOMP",
    "k00069a21HOT",
    "k000a1940COMP",
    "k00069d10HOT",
    "k000a12d8COMP",
    "k00069a32HOT",
    "k000a1869COMP",
    "k00069cb1HOT",
    "k000a16f8COMP",
    "k00069c05HOT",
    "k000a1619COMP",
    "k00069bbcHOT",
    "k000a1860COMP",
    "k00069cabHOT",
    "k000a1550COMP",
    "k00069b5fHOT",
    "k000a1a07COMP",
    "k00069d73HOT",
    "k000a1455COMP",
    "k00069ae9HOT",
    "k000a1867COMP",
    "k00069cb0HOT",
    "k000a128fCOMP",
    "k00069a06HOT",
    "k000a158cCOMP",
    "k00069b80HOT",
    "k000a1798COMP",
    "k00069c49HOT",
    "k000a1914COMP",
    "k00069d02HOT",
    "k000a178cCOMP",
    "k00069c42HOT",
    "k000a1549COMP",
    "k00069b59HOT",
    "k000a16e8COMP",
    "k00069c03HOT",
    "k000a19c1COMP",
    "k00069d4eHOT",
    "k000a18bcCOMP",
    "k00069cdaHOT",
    "k000a1379COMP",
    "k00069a7dHOT",
    "k000a13ffCOMP",
    "k00069abdHOT",
    "k000a18b2COMP",
    "k00069cd7HOT",
    "k000a125eCOMP",
    "k000699f4HOT",
    "k000a1570COMP",
    "k00069b70HOT",
    "k000a171fCOMP",
    "k00069c17HOT",
    "k000a19a5COMP",
    "k00069d3eHOT",
    "k000a14c5COMP",
    "k00069b21HOT",
    "k000a12aeCOMP",
    "k00069a17HOT",
    "k000a1403COMP",
    "k00069ac0HOT",
    "k000a1983COMP",
    "k00069d2cHOT",
    "k000a1520COMP",
    "k00069b48HOT",
    "k000a1563COMP",
    "k00069b6bHOT",
    "k000a26cfCOMP",
    "k0006bfb6HOT",
    "k000a2705COMP",
    "k0006bfb7HOT",
    "k000a2756COMP",
    "k0006bfb8HOT",
    "k000a2799COMP",
    "k0006bfb9HOT",
    "k000a27e1COMP",
    "k0006bfbaHOT",
    "k000a28c3COMP",
    "k0006bfbbHOT",
    "k000a28f1COMP",
    "k0006bfbcHOT",
    "k000a1ad2COMP",
    "k0006bf8cHOT",
    "k000a1ad3COMP",
    "k0006bf8dHOT",
    "k000a1b4fCOMP",
    "k0006bf8eHOT",
    "k000a1b50COMP",
    "k0006bf8fHOT",
    "k000a1b5eCOMP",
    "k0006bf90HOT",
    "k000a1b5fCOMP",
    "k0006bf91HOT",
    "k000a1bbcCOMP",
    "k0006bf92HOT",
    "k000a1bbdCOMP",
    "k0006bf93HOT",
    "k000a1c07COMP",
    "k0006bf94HOT",
    "k000a1d11COMP",
    "k0006bf95HOT",
    "k000a1d88COMP",
    "k0006bf96HOT",
    "k000a1dc4COMP",
    "k0006bf97HOT",
    "k000a1dc5COMP",
    "k0006bf98HOT",
    "k000a1de4COMP",
    "k0006bf99HOT",
    "k000a1de8COMP",
    "k0006bf9aHOT",
    "k000a1de9COMP",
    "k0006bf9bHOT",
    "k000a1e25COMP",
    "k0006bf9cHOT",
    "k000a1e53COMP",
    "k0006bf9dHOT",
    "k000a1e9dCOMP",
    "k0006bf9eHOT",
    "k000a1f0cCOMP",
    "k0006bf9fHOT",
    "k000a1f19COMP",
    "k0006bfa0HOT",
    "k000a1fbaCOMP",
    "k0006bfa1HOT",
    "k000a2002COMP",
    "k0006bfa2HOT",
    "k000a200aCOMP",
    "k0006bfa3HOT",
    "k000a2097COMP",
    "k0006bfa4HOT",
    "k000a2098COMP",
    "k0006bfa5HOT",
    "k000a20d4COMP",
    "k0006bfa6HOT",
    "k000a216bCOMP",
    "k0006bfa7HOT",
    "k000a21afCOMP",
    "k0006bfa8HOT",
    "k000a2323COMP",
    "k0006bfa9HOT",
    "k000a2419COMP",
    "k0006bfaaHOT",
    "k000a249cCOMP",
    "k0006bfabHOT",
    "k000a24ecCOMP",
    "k0006bfacHOT",
    "k000a24f3COMP",
    "k0006bfadHOT",
    "k000a253dCOMP",
    "k0006bfaeHOT",
    "k000a2588COMP",
    "k0006bfafHOT",
    "k000a25c3COMP",
    "k0006bfb0HOT",
    "k000a25e1COMP",
    "k0006bfb1HOT",
    "k000a263fCOMP",
    "k0006bfb2HOT",
    "k000a266bCOMP",
    "k0006bfb4HOT",
    "k000a2682COMP",
    "k0006bfb5HOT",
    "k000a1c7cCOMP",
    "k00069e5fHOT",
    "k000a2385COMP",
    "k0006a07aHOT",
    "k000a1c12COMP",
    "k00069e3fHOT",
    "k000a1d0aCOMP",
    "k00069e93HOT",
    "k000a1b1cCOMP",
    "k00069deaHOT",
    "k000a1ab1COMP",
    "k00069db7HOT",
    "k000a1be3COMP",
    "k00069e2aHOT",
    "k000a1e1bCOMP",
    "k00069ef4HOT",
    "k000a240eCOMP",
    "k0006a099HOT",
    "k000a1fadCOMP",
    "k00069f6fHOT",
    "k000a1facCOMP",
    "k00069f6eHOT",
    "k000a2019COMP",
    "k00069f93HOT",
    "k000a1f8cCOMP",
    "k00069f63HOT",
    "k000a2041COMP",
    "k00069fa2HOT",
    "k000a1be4COMP",
    "k00069e2bHOT",
    "k000a1fdaCOMP",
    "k00069f80HOT",
    "k000a208dCOMP",
    "k00069fb5HOT",
    "k000a1b2cCOMP",
    "k00069df0HOT",
    "k000a2070COMP",
    "k00069fadHOT",
    "k000a214fCOMP",
    "k00069fe8HOT",
    "k000a2000COMP",
    "k00069f8aHOT",
    "k000a21e9COMP",
    "k0006a013HOT",
    "k000a1d09COMP",
    "k00069e92HOT",
    "k000a1cfcCOMP",
    "k00069e8dHOT",
    "k000a22c2COMP",
    "k0006a04aHOT",
    "k000a27a6COMP",
    "k0006a147HOT",
    "k000a2614COMP",
    "k0006a109HOT",
    "k000a1be8COMP",
    "k00069e2cHOT",
    "k000a26aaCOMP",
    "k0006a121HOT",
    "k000a1a65COMP",
    "k00069d9cHOT",
    "k000a1e63COMP",
    "k00069f0dHOT",
    "k000a1ed9COMP",
    "k00069f2dHOT",
    "k000a27a5COMP",
    "k0006a146HOT",
    "k000a213eCOMP",
    "k00069fe5HOT",
    "k000a2274COMP",
    "k0006a034HOT",
    "k000a1e4aCOMP",
    "k00069f04HOT",
    "k000a2440COMP",
    "k0006a0a8HOT",
    "k000a213dCOMP",
    "k00069fe4HOT",
    "k000a1c2fCOMP",
    "k00069e4eHOT",
    "k000a1cb6COMP",
    "k00069e75HOT",
    "k000a1b92COMP",
    "k00069e11HOT",
    "k000a1e7cCOMP",
    "k00069f14HOT",
    "k000a226cCOMP",
    "k0006a031HOT",
    "k000a1c67COMP",
    "k00069e5aHOT",
    "k000a1d86COMP",
    "k00069ebcHOT",
    "k000a2613COMP",
    "k0006a108HOT",
    "k000a1f9eCOMP",
    "k00069f69HOT",
    "k000a22c1COMP",
    "k0006a049HOT",
    "k000a1cb9COMP",
    "k00069e77HOT",
    "k000a1b2bCOMP",
    "k00069defHOT",
    "k000a25d2COMP",
    "k0006a0faHOT",
    "k000a2620COMP",
    "k0006a10cHOT",
    "k000a1c11COMP",
    "k00069e3eHOT",
    "k000a26abCOMP",
    "k0006a122HOT",
    "k000a1af6COMP",
    "k00069dd7HOT",
    "k000a1f3dCOMP",
    "k00069f4fHOT",
    "k000a2345COMP",
    "k0006a06bHOT",
    "k000a21c8COMP",
    "k0006a00bHOT",
    "k000a1b8cCOMP",
    "k00069e0fHOT",
    "k000a1c62COMP",
    "k00069e59HOT",
    "k000a21a9COMP",
    "k0006a004HOT",
    "k000a2423COMP",
    "k0006a0a0HOT",
    "k000a2590COMP",
    "k0006a0ebHOT",
    "k000a26d2COMP",
    "k0006a129HOT",
    "k000a2567COMP",
    "k0006a0e5HOT",
    "k000a1b8dCOMP",
    "k00069e10HOT",
    "k000a1e4bCOMP",
    "k00069f05HOT",
    "k000a2549COMP",
    "k0006a0dfHOT",
    "k000a1dbdCOMP",
    "k00069ed0HOT",
    "k000a1af5COMP",
    "k00069dd6HOT",
    "k000a23ddCOMP",
    "k0006a08cHOT",
    "k000a2531COMP",
    "k0006a0ddHOT",
    "k000a20ceCOMP",
    "k00069fc9HOT",
    "k000a2611COMP",
    "k0006a107HOT",
    "k000a238bCOMP",
    "k0006a07cHOT",
    "k000a201aCOMP",
    "k00069f94HOT",
    "k000a1d93COMP",
    "k00069ec0HOT",
    "k000a1dfeCOMP",
    "k00069eeaHOT",
    "k000a1cb8COMP",
    "k00069e76HOT",
    "k000a1b1bCOMP",
    "k00069de9HOT",
    "k000a1d5dCOMP",
    "k00069eaeHOT",
    "k000a1f38COMP",
    "k00069f4dHOT",
    "k000a1be9COMP",
    "k00069e2dHOT",
    "k000a1e9fCOMP",
    "k00069f1cHOT",
    "k000a1ab0COMP",
    "k00069db6HOT",
    "k000a2530COMP",
    "k0006a0dcHOT",
    "k000a1c7bCOMP",
    "k00069e5eHOT",
    "k000a2909COMP",
    "k0006a16eHOT",
    "k000a2674COMP",
    "k0006a11bHOT",
    "k000a23dcCOMP",
    "k0006a08bHOT",
    "k000a1d40COMP",
    "k00069ea8HOT",
    "k000a1dfdCOMP",
    "k00069ee9HOT",
    "k000a26c4COMP",
    "k0006a127HOT",
    "k000a2641COMP",
    "k0006a110HOT",
    "k000a2640COMP",
    "k000a249dCOMP",
    "k000a221dCOMP",
    "k000a24edCOMP",
    "k000a2900COMP",
    "k000a216cCOMP",
    "k000a23eaCOMP",
    "k000a282aCOMP",
    "k000a1f0dCOMP",
    "k000a26b4COMP",
    "k000a279cCOMP",
    "k000a282cCOMP",
    "k000a28c4COMP",
    "k000a2744COMP",
    "k000a20d1COMP",
    "k000a20eaCOMP",
    "k000a200bCOMP",
    "k000a15adCOMP",
    "k000a2640COMP",
    "k000a1fdbCOMP",
    "k000a26afCOMP",
    "k000a2997COMP",
    "k000a26aeCOMP",
    "k000a2568COMP",
    "k000a1c86COMP",
    "k000a2071COMP",
    "k000a1b97COMP",
    "k000a28f3COMP",
    "k000a15a6COMP",
    "k000a1d87COMP",
    "k000a1ebeCOMP",
    "k000a1dbeCOMP",
    "k000a254aCOMP",
    "k000a1239COMP",
    "k000a1f6dCOMP",
    "k000a1a66COMP",
    "k000a27e4COMP",
    "k000a2089COMP",
    "k000a164cCOMP",
    "k000a2042COMP",
    "k000a2998COMP",
    "k000a2958COMP",
    "k000a2115COMP",
    "k000a1bf2COMP",
    "k000a2469COMP",
    "k000a244aCOMP",
    "k000a24f5COMP",
    "k000a2727COMP",
    "k000a1f9fCOMP",
    "k000a2675COMP",
    "k000a287bCOMP",
    "k000a26bdCOMP",
    "k000a14beCOMP",
    "k000a27e8COMP",
    "k000a26deCOMP",
    "k000a1c63COMP",
    "k000a2875COMP",
    "k000a297dCOMP",
    "k000a228fCOMP",
    "k000a2438COMP",
    "k000a2421COMP",
    "k000a22e9COMP",
    "k000a26afCOMP",
    "k000a2982COMP",
    "k000a1b88COMP",
    "k000a2990COMP",
    "k000a2591COMP",
    "k000a27e3COMP",
    "k000a1f07COMP",
    "k000a2731COMP",
    "k000a288bCOMP",
    "k000a1e4eCOMP",
    "k000a1576COMP",
    "k000a1cecCOMP",
    "k000a1e64COMP",
    "k000a1f3eCOMP",
    "k000a208eCOMP",
    "k000a1d8bCOMP",
    "k000a1c68COMP",
    "k000a140fCOMP",
    "k000a2758COMP",
    "k000a238cCOMP",
    "k000a192fCOMP",
    "k00069d09HOT",
    "k000a1354COMP",
    "k00069a68HOT",
    "k000a0cd1COMP",
    "e0001a6bbHOT",
    "k000a0cd2COMP",
    "e0001a6bcHOT",
    "k000a0d28COMP",
    "e0001a589HOT",
    "k000a0d29COMP",
    "e0001a58aHOT",
    "k000a0dc5COMP",
    "e0001a5d2HOT",
    "k000a0dc4COMP",
    "e0001a5d1HOT",
    "k000a0bcfCOMP",
    "e0001a62fHOT",
    "k000a0bd0COMP",
    "e0001a630HOT",
    "k000a0c3cCOMP",
    "e0001a67aHOT",
    "k000a0c3bCOMP",
    "e0001a679HOT",
    "k000a0c5fCOMP",
    "e0001a689HOT",
    "k000a0c60COMP",
    "e0001a68aHOT",
    "k000a0ca2COMP",
    "e0001a6acHOT",
    "k000a0ca1COMP",
    "e0001a6abHOT",
    "k000a1931COMP",
    "k00069d0aHOT",
    "k000a1930COMP",
    "k00069d08HOT",
    "k000a1356COMP",
    "k00069a69HOT",
    "k000a1355COMP",
    "k00069a67HOT",
    "k000a15a9COMP",
    "k0006bf72HOT",
    "k000a15aaCOMP",
    "k0006bf71HOT",
    "k000a15a8COMP",
    "k0006bf73HOT",
    "k000a14bbCOMP",
    "k00069b1bHOT",
    "k000a14baCOMP",
    "k00069b1cHOT",
    "k000a14b9COMP",
    "k00069b1dHOT",
    "k000a1236COMP",
    "k000699dbHOT",
    "k000a1234COMP",
    "k000699ddHOT",
    "k000a1235COMP",
    "k000699dcHOT",
    "k000a1573COMP",
    "k00069b71HOT",
    "k000a1572COMP",
    "k00069b72HOT",
    "k000a1571COMP",
    "k00069b73HOT",
    "k000a15a1COMP",
    "k00069b8eHOT",
    "k000a15a3COMP",
    "k00069b8cHOT",
    "k000a15a2COMP",
    "k00069b8dHOT",
    "k000a1649COMP",
    "k00069bc9HOT",
    "k000a1647COMP",
    "k00069bcbHOT",
]



from peek_plugin_diagram._private.storage.Lookups import DispColor
from peek_plugin_diagram._private.storage.Lookups import DispLineStyle
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase
from typing import Optional


class ShapeEllipse(ShapeBase):
    @staticmethod
    def fillColor(disp) -> DispColor:
        return disp.get("fcl")

    @staticmethod
    def lineColor(disp) -> DispColor:
        return disp.get("lcl")

    @staticmethod
    def lineStyle(disp) -> DispLineStyle:
        return disp.get("lsl")

    @staticmethod
    def lineWidth(disp) -> Optional[float]:
        return float(disp.get("w")) if disp.get("w") else None

    @staticmethod
    def centerPointX(disp) -> float:
        return disp.get("g")[0]

    @staticmethod
    def centerPointY(disp) -> float:
        return disp.get("g")[1]

    @staticmethod
    def center(disp) -> Point:
        return Point(
            x=ShapeEllipse.centerPointX(disp), y=ShapeEllipse.centerPointY(disp)
        )

    @staticmethod
    def xRadius(disp) -> float:
        return disp.get("xr")

    @staticmethod
    def yRadius(disp) -> float:
        return disp.get("yr")

    @staticmethod
    def rotation(disp) -> float:
        return disp.get("r")

    @staticmethod
    def startAngle(disp) -> float:
        """

        :param disp: a dict
        :return: startAngle in degrees
        """
        return disp.get("sa")

    @staticmethod
    def endAngle(disp) -> float:
        """

        :param disp: a dict
        :return:  endAngle in degrees
        """
        return disp.get("ea")



"""Added CoordSet dispGroup enabled

Peek Plugin Database Migration Script

Revision ID: f4dce3e782ec
Revises: 81c1d2809046
Create Date: 2019-05-17 14:37:07.220197

"""

# revision identifiers, used by Alembic.
revision = "f4dce3e782ec"
down_revision = "81c1d2809046"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.add_column(
        "ModelCoordSet",
        sa.Column(
            "dispGroupTemplatesEnabled",
            sa.Boolean(),
            server_default="false",
            nullable=True,
        ),
        schema="pl_diagram",
    )

    op.execute(
        'UPDATE "pl_diagram"."ModelCoordSet" SET "dispGroupTemplatesEnabled" = false '
    )

    op.alter_column(
        "ModelCoordSet",
        "dispGroupTemplatesEnabled",
        type_=sa.Boolean(),
        nullable=False,
        schema="pl_diagram",
    )


def downgrade():
    op.drop_column("ModelCoordSet", "dispGroupTemplatesEnabled", schema="pl_diagram")



from typing import List

from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import Point
from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase


class ShapeNull(ShapeBase):
    @staticmethod
    def geom(disp) -> List[float]:
        return disp["g"]

    @staticmethod
    def centerPointX(disp) -> float:
        return disp["g"][0]

    @staticmethod
    def centerPointY(disp) -> float:
        return disp["g"][1]

    @staticmethod
    def center(disp) -> Point:
        return Point(
            x=ShapeNull.centerPointX(disp), y=ShapeNull.centerPointY(disp)
        )



import json
from typing import List

from peek_plugin_diagram.worker.canvas_shapes.ShapeBase import ShapeBase


class ShapeGroup(ShapeBase):
    @staticmethod
    def items(disp) -> List[dict]:
        if disp["di"] is None:
            return []

        return json.loads(disp["di"])

    @staticmethod
    def groupName(disp) -> str:
        return disp["n"]



from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram.tuples.ColorUtil import invertColor


@addTupleType
class ShapeColorTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "ShapeColorTuple"

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()

    name: str = TupleField()
    darkColor: str = TupleField()
    lightColor: str = TupleField()
    altColor: str = TupleField()
    swapPeriod: float = TupleField()

    modelSetKey: str = TupleField()

    importHash: str = TupleField()

    showForEdit: bool = TupleField()

    blockApiUpdate: bool = TupleField()

    @property
    def color(self):
        return self.darkColor

    @color.setter
    def color(self, value: str):
        self.darkColor = value
        if value:
            self.lightColor = invertColor(value, "#fff")



import json

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ShapeLineStyleTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "ShapeLineStyleTuple"

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()

    name: str = TupleField()
    backgroundFillDashSpace: bool = TupleField()
    capStyle: str = TupleField()
    joinStyle: str = TupleField()
    dashPattern: str = TupleField()
    startArrowSize: int = TupleField()
    endArrowSize: int = TupleField()
    winStyle: int = TupleField()

    modelSetKey: str = TupleField()

    importHash: str = TupleField()
    scalable: bool = TupleField()

    showForEdit: bool = TupleField()
    blockApiUpdate: bool = TupleField()

    @property
    def dashPatternParsed(self):
        if self.dashPattern is None:
            return None
        return json.loads(self.dashPattern)



import logging
from collections import defaultdict
from typing import Dict
from typing import List

from vortex.Tuple import Tuple

from peek_plugin_diagram._private.storage.Lookups import _Hasher
from peek_plugin_diagram.tuples.lookup_tuples.ShapeLayerTuple import (
    ShapeLayerTuple,
)
from peek_plugin_diagram.tuples.lookup_tuples.ShapeLevelTuple import (
    ShapeLevelTuple,
)
from peek_plugin_diagram.worker.WorkerDiagramLookupApi import (
    WorkerDiagramLookupApi,
)

logger = logging.getLogger(__name__)


_hasher = _Hasher()


class ShapeLookupLinker:
    def __init__(self):
        self._isLoaded = False

        self._levelsById = {}
        self._layersById = {}
        self._colorById = {}
        self._textStyleById = {}
        self._lineStyleById = {}

        self._linkedLevelIds = set()
        self._linkedLayerIds = set()

        self._shapeIndexSetByLevelId = defaultdict(set)
        self._shapeIndexSetByLayerId = defaultdict(set)

        self.loadLookups()

    def loadLookups(self):
        if not self._isLoaded:
            self._loadAllLookups()
            self._isLoaded = True

    def levelsOrderedByOrder(
        self, coordSetKey: str, linkedOnly: bool = False
    ) -> List[ShapeLevelTuple]:
        levels = []
        for level in self._levelsById.values():
            if linkedOnly and level.id not in self._linkedLevelIds:
                continue
            if level.data.get("coordSetKey") == coordSetKey:
                levels.append(level)

        levels.sort(key=lambda l: l.order)

        return levels

    @property
    def shapeIndexSetByLevelId(self):
        return self._shapeIndexSetByLevelId

    def layersOrderedByOrder(
        self, modelSetKey: str, linkedOnly: bool = False
    ) -> List[ShapeLayerTuple]:
        layers = []
        for layer in self._layersById.values():
            if linkedOnly and layer.id not in self._linkedLayerIds:
                continue
            if layer.data.get("modelSetKey") == modelSetKey:
                layers.append(layer)

        layers.sort(key=lambda l: l.order)

        return layers

    @property
    def shapeIndexSetByLayerId(self):
        return self._shapeIndexSetByLayerId

    def _loadAllLookups(self):
        levels = WorkerDiagramLookupApi.getLevels()
        self._levelsById = self._makeLookupMap(levels)

        colors = WorkerDiagramLookupApi.getColors()
        self._colorById = self._makeLookupMap(colors)

        lineStyles = WorkerDiagramLookupApi.getLineStyles()
        self._lineStyleById = self._makeLookupMap(lineStyles)

        textStyles = WorkerDiagramLookupApi.getTextStyles()
        self._textStyleById = self._makeLookupMap(textStyles)

        layers = WorkerDiagramLookupApi.getLayers()
        self._layersById = self._makeLookupMap(layers)

    def _makeLookupMap(
        self,
        lookups: List[Tuple],
    ) -> Dict[str, Tuple]:
        mapped = {}

        for lookup in lookups:
            key = str(getattr(lookup, "key"))
            id_ = str(_hasher.decode(key)[0])
            mapped[id_] = lookup

        return mapped

    def linkShapesToLookups(self, shapes: List[Dict]) -> List[Dict]:
        self._linkedLevelIds = set()
        self._linkedLayerIds = set()

        self._shapeIndexSetByLevelId = defaultdict(set)
        self._shapeIndexSetByLayerId = defaultdict(set)

        for shapeIndex, shape in enumerate(shapes, 0):
            try:
                linkedLevelKey, linkedLayerKey = self._linkShapeLookups(shape)

                if linkedLevelKey:
                    self._linkedLevelIds.add(linkedLevelKey)
                    self._shapeIndexSetByLevelId[linkedLevelKey].add(shapeIndex)
                if linkedLayerKey:
                    self._linkedLayerIds.add(linkedLayerKey)
                    self._shapeIndexSetByLayerId[linkedLayerKey].add(shapeIndex)
            except KeyError as e:
                logger.error(shape)
                logger.exception(e)

        return shapes

    def _linkShapeLookups(self, shape: Dict):
        linkedLevelId = None
        linkedLayerId = None

        if le := shape.get("le"):
            linkedLevelId = le
            le = str(le)
            shape["lel"] = self._levelsById.get(le)

        if la := shape.get("la"):
            linkedLayerId = la
            la = str(la)
            shape["lal"] = self._layersById.get(la)

        if fs := shape.get("fs"):
            fs = str(fs)
            shape["fsl"] = self._textStyleById.get(fs)

        if c := shape.get("c"):
            c = str(c)
            shape["cl"] = self._colorById.get(c)

        if bc := shape.get("bc"):
            bc = str(bc)
            shape["bcl"] = self._colorById.get(bc)

        if lc := shape.get("lc"):
            lc = str(lc)
            shape["lcl"] = self._colorById.get(lc)

        if ec := shape.get("ec"):
            ec = str(ec)
            shape["ecl"] = self._colorById.get(ec)

        if fc := shape.get("fc"):
            fc = str(fc)
            shape["fcl"] = self._colorById.get(fc)

        if ls := shape.get("ls"):
            ls = str(ls)
            shape["lsl"] = self._lineStyleById.get(ls)

        return linkedLevelId, linkedLayerId



""" Updates for DispGroup

Peek Plugin Database Migration Script

Revision ID: 8ef78f862ac8
Revises: 1b17ef0cca17
Create Date: 2019-02-17 13:45:55.038615

"""

# revision identifiers, used by Alembic.
revision = "8ef78f862ac8"
down_revision = "1b17ef0cca17"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    op.add_column(
        "DispGroup",
        sa.Column(
            "compileAsTemplate", sa.Boolean(), server_default="false", nullable=False
        ),
        schema="pl_diagram",
    )

    op.drop_constraint(
        "DispGroupPointer_groupId_fkey",
        "DispGroupPointer",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.drop_index(
        "idxDispGroupPointer_groupId",
        table_name="DispGroupPointer",
        schema="pl_diagram",
    )
    op.alter_column(
        "DispGroupPointer",
        "groupId",
        new_column_name="targetDispGroupId",
        type_=sa.INTEGER(),
        nullable=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idxDispGroupPointer_targetDispGroupId",
        "DispGroupPointer",
        ["targetDispGroupId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_foreign_key(
        None,
        "DispGroupPointer",
        "DispGroup",
        ["targetDispGroupId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="SET NULL",
    )

    op.add_column(
        "DispBase",
        sa.Column("groupId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_groupId", "DispBase", ["groupId"], unique=False, schema="pl_diagram"
    )
    op.create_foreign_key(
        None,
        "DispBase",
        "DispBase",
        ["groupId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="CASCADE",
    )

    op.drop_table("DispGroupItem", schema="pl_diagram")


def downgrade():
    raise NotImplementedError()



"""Initial

Peek Plugin Database Migration Script

Revision ID: 7eed363f5df5
Revises: 
Create Date: 2017-07-13 23:07:17.930374

"""

# revision identifiers, used by Alembic.
from sqlalchemy import func
from sqlalchemy.dialects.mssql.base import MSDialect
from sqlalchemy.dialects.postgresql.base import PGDialect

revision = "7eed363f5df5"
down_revision = None
branch_labels = None
depends_on = None

from sqlalchemy.schema import Sequence, CreateSequence, DropSequence

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def isMssqlDialect():
    return isinstance(op.get_bind().engine.dialect, MSDialect)


def isPostGreSQLDialect():
    return isinstance(op.get_bind().engine.dialect, PGDialect)


def nextval(seqName):
    if isMssqlDialect():
        return sa.text('(NEXT VALUE FOR "pl_diagram"."%s")' % seqName)

    return sa.text("nextval('pl_diagram.\"%s\"')" % seqName)


def upgrade():
    op.execute(
        CreateSequence(Sequence("DispBase_id_seq", minvalue=0, schema="pl_diagram"))
    )
    op.execute(
        CreateSequence(
            Sequence("LiveDbDispLink_id_seq", minvalue=0, schema="pl_diagram")
        )
    )

    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "DispCompilerQueue",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("dispId", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id", "dispId"),
        schema="pl_diagram",
    )
    op.create_table(
        "ModelSet",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_ModelSet_name", "ModelSet", ["name"], unique=True, schema="pl_diagram"
    )
    op.create_table(
        "Setting",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_table(
        "DispColor",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column(
            "color", sa.String(length=20), server_default="orange", nullable=True
        ),
        sa.Column("altColor", sa.String(length=20), nullable=True),
        sa.Column("swapPeriod", sa.Float(), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispColor_importHash",
        "DispColor",
        ["importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispColor_modelSetId",
        "DispColor",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispLayer",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("order", sa.Integer(), server_default="0", nullable=False),
        sa.Column("selectable", sa.Boolean(), server_default="false", nullable=False),
        sa.Column("visible", sa.Boolean(), server_default="true", nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLayer_modelSetId",
        "DispLayer",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispLineStyle",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column(
            "backgroundFillDashSpace",
            sa.Boolean(),
            server_default="false",
            nullable=False,
        ),
        sa.Column("capStyle", sa.String(length=15), nullable=False),
        sa.Column("joinStyle", sa.String(length=15), nullable=False),
        sa.Column("dashPattern", sa.String(length=50), nullable=True),
        sa.Column("startArrowSize", sa.Integer(), nullable=True),
        sa.Column("endArrowSize", sa.Integer(), nullable=True),
        sa.Column("winStyle", sa.Integer(), nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLineStyle_importHash",
        "DispLineStyle",
        ["importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLineStyle_modelSetId",
        "DispLineStyle",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispTextStyle",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column(
            "fontName", sa.String(length=30), server_default="GillSans", nullable=False
        ),
        sa.Column("fontSize", sa.Integer(), server_default="9", nullable=False),
        sa.Column("fontStyle", sa.String(length=30), nullable=True),
        sa.Column("scalable", sa.Boolean(), server_default="true", nullable=False),
        sa.Column("scaleFactor", sa.Integer(), server_default="1", nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispTextStyle_importHash",
        "DispTextStyle",
        ["importHash"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispTextStyle_modelSetId",
        "DispTextStyle",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "ModelConnType",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnTypeModelSetId",
        "ModelConnType",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "ModelCoordSet",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("initialPanX", sa.Float(), server_default="0", nullable=False),
        sa.Column("initialPanY", sa.Float(), server_default="0", nullable=False),
        sa.Column("initialZoom", sa.Float(), server_default="0", nullable=False),
        sa.Column("enabled", sa.Boolean(), server_default="false", nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("importId1", sa.Integer(), nullable=True),
        sa.Column("importId2", sa.String(length=100), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordModelSetId",
        "ModelCoordSet",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordSetImportId1",
        "ModelCoordSet",
        ["importId1"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordSetImportId2",
        "ModelCoordSet",
        ["importId2"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxCoordSetModelName",
        "ModelCoordSet",
        ["modelSetId", "name"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_table(
        "ModelNodeType",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeTypeModelSetId",
        "ModelNodeType",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "SettingProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("settingId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=16), nullable=True),
        sa.Column("int_value", sa.Integer(), nullable=True),
        sa.Column("char_value", sa.String(length=7500), nullable=True),
        sa.Column("boolean_value", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(
            ["settingId"],
            ["pl_diagram.Setting.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_SettingProperty_settingId",
        "SettingProperty",
        ["settingId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispLevel",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("order", sa.Integer(), server_default="0", nullable=False),
        sa.Column("minZoom", sa.Float(), nullable=True),
        sa.Column("maxZoom", sa.Float(), nullable=True),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispLevel_coordSetId",
        "DispLevel",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "GridKeyCompilerQueue",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("gridKey", sa.String(length=30), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "gridKey", "coordSetId"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GKCompQueue_coordSetId_gridKey",
        "GridKeyCompilerQueue",
        ["coordSetId", "gridKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "GridKeyIndexCompiled",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("gridKey", sa.String(length=30), nullable=False),
        sa.Column("blobData", sa.LargeBinary(), nullable=False),
        sa.Column("lastUpdate", sa.String(length=50), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GKIndexUpdate_coordSetId",
        "GridKeyIndexCompiled",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GKIndexUpdate_gridKey",
        "GridKeyIndexCompiled",
        ["gridKey"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_table(
        "ModelNode",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("propsJson", sa.String(length=500), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importId1", sa.Integer(), nullable=True),
        sa.Column("importId2", sa.String(length=100), nullable=True),
        sa.Column("typeId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["typeId"], ["pl_diagram.ModelNodeType.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeImportId1",
        "ModelNode",
        ["importId1"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeImportId2",
        "ModelNode",
        ["importId2"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeModelSetId",
        "ModelNode",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeTypeId", "ModelNode", ["typeId"], unique=False, schema="pl_diagram"
    )
    op.create_table(
        "DispBase",
        sa.Column(
            "id",
            sa.Integer(),
            server_default=nextval("DispBase_id_seq"),
            nullable=False,
        ),
        sa.Column("type", sa.Integer(), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.Column("layerId", sa.Integer(), nullable=True),
        sa.Column("levelId", sa.Integer(), nullable=True),
        sa.Column("dispJson", sa.String(length=200000), nullable=True),
        sa.Column("importUpdateDate", sa.DateTime(), nullable=True),
        sa.Column("importHash", sa.String(length=100), nullable=True),
        sa.Column("importGroupHash", sa.String(length=100), nullable=True),
        sa.ForeignKeyConstraint(
            ["coordSetId"], ["pl_diagram.ModelCoordSet.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["layerId"], ["pl_diagram.DispLayer.id"]),
        sa.ForeignKeyConstraint(["levelId"], ["pl_diagram.DispLevel.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_coordSetId_",
        "DispBase",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_importGroupHash",
        "DispBase",
        ["importGroupHash"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_importUpdateDate",
        "DispBase",
        ["importUpdateDate"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_layerId", "DispBase", ["layerId"], unique=False, schema="pl_diagram"
    )
    op.create_index(
        "idx_Disp_levelId", "DispBase", ["levelId"], unique=False, schema="pl_diagram"
    )
    op.create_table(
        "ModelConn",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("propsJson", sa.String(length=500), nullable=True),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("importId1", sa.Integer(), nullable=True),
        sa.Column("importId2", sa.String(length=100), nullable=True),
        sa.Column("typeId", sa.Integer(), nullable=False),
        sa.Column("srcId", sa.Integer(), nullable=False),
        sa.Column("dstId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dstId"], ["pl_diagram.ModelNode.id"]),
        sa.ForeignKeyConstraint(
            ["modelSetId"], ["pl_diagram.ModelSet.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["srcId"], ["pl_diagram.ModelNode.id"]),
        sa.ForeignKeyConstraint(["typeId"], ["pl_diagram.ModelConnType.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnImportId1",
        "ModelConn",
        ["importId1"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnImportId2",
        "ModelConn",
        ["importId2"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnModelDstId", "ModelConn", ["dstId"], unique=False, schema="pl_diagram"
    )
    op.create_index(
        "idxConnModelSetId",
        "ModelConn",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnModelSrcId", "ModelConn", ["srcId"], unique=False, schema="pl_diagram"
    )
    op.create_index(
        "idxConnModelTypeId", "ModelConn", ["typeId"], unique=False, schema="pl_diagram"
    )
    op.create_index(
        "idxConnSrcDst",
        "ModelConn",
        ["srcId", "dstId"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_table(
        "DispEllipse",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("xRadius", sa.Float(), server_default="10.0", nullable=False),
        sa.Column("yRadius", sa.Float(), server_default="10.0", nullable=False),
        sa.Column("rotation", sa.Float(), server_default="0", nullable=False),
        sa.Column("startAngle", sa.Float(), server_default="0", nullable=False),
        sa.Column("endAngle", sa.Float(), server_default="360", nullable=False),
        sa.Column("lineWidth", sa.Integer(), server_default="2", nullable=False),
        sa.Column("geomJson", sa.String(length=2000), nullable=False),
        sa.Column("fillColorId", sa.Integer(), nullable=True),
        sa.Column("lineColorId", sa.Integer(), nullable=True),
        sa.Column("lineStyleId", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["fillColorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["lineColorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["lineStyleId"], ["pl_diagram.DispLineStyle.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_fillColorId",
        "DispEllipse",
        ["fillColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_lineColorId",
        "DispEllipse",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_lineStyleId",
        "DispEllipse",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispGroup",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
        schema="pl_diagram",
    )
    op.create_table(
        "DispPolygon",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("cornerRadius", sa.Float(), server_default="0", nullable=False),
        sa.Column("lineWidth", sa.Integer(), server_default="2", nullable=False),
        sa.Column("geomJson", sa.String(length=200000), nullable=False),
        sa.Column("fillColorId", sa.Integer(), nullable=True),
        sa.Column("fillDirection", sa.Integer(), nullable=True),
        sa.Column("fillPercent", sa.Float(), nullable=True),
        sa.Column("lineColorId", sa.Integer(), nullable=True),
        sa.Column("lineStyleId", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["fillColorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["lineColorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["lineStyleId"], ["pl_diagram.DispLineStyle.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolygon_fillColorId",
        "DispPolygon",
        ["fillColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolygon_lineColorId",
        "DispPolygon",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolygon_lineStyleId",
        "DispPolygon",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispPolyline",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("lineWidth", sa.Integer(), server_default="2", nullable=False),
        sa.Column("geomJson", sa.String(length=200000), nullable=False),
        sa.Column("lineColorId", sa.Integer(), nullable=True),
        sa.Column("lineStyleId", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["lineColorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["lineStyleId"], ["pl_diagram.DispLineStyle.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_lineColorId",
        "DispPolyline",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_lineStyleId",
        "DispPolyline",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispText",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("verticalAlign", sa.Integer(), server_default="-1", nullable=False),
        sa.Column("horizontalAlign", sa.Integer(), server_default="0", nullable=False),
        sa.Column("rotation", sa.Float(), server_default="0", nullable=False),
        sa.Column(
            "text",
            sa.String(length=1000),
            server_default="new text label",
            nullable=False,
        ),
        sa.Column("textFormat", sa.String(length=1000), nullable=True),
        sa.Column("geomJson", sa.String(length=2000), nullable=False),
        sa.Column("colorId", sa.Integer(), nullable=True),
        sa.Column("textStyleId", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["colorId"], ["pl_diagram.DispColor.id"]),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["textStyleId"], ["pl_diagram.DispTextStyle.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispText_colorId",
        "DispText",
        ["colorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispText_styleId",
        "DispText",
        ["textStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "GridKeyIndex",
        sa.Column("gridKey", sa.String(length=30), nullable=False),
        sa.Column("dispId", sa.Integer(), nullable=False),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["coordSetId"], ["pl_diagram.ModelCoordSet.id"]),
        sa.ForeignKeyConstraint(
            ["dispId"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("gridKey", "dispId"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GridKeyIndex_coordSetId",
        "GridKeyIndex",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GridKeyIndex_dispId",
        "GridKeyIndex",
        ["dispId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_GridKeyIndex_gridKey",
        "GridKeyIndex",
        ["gridKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "LiveDbDispLink",
        sa.Column(
            "id",
            sa.Integer(),
            server_default=nextval("LiveDbDispLink_id_seq"),
            nullable=False,
        ),
        sa.Column("coordSetId", sa.Integer(), nullable=False),
        sa.Column("dispId", sa.Integer(), nullable=False),
        sa.Column("dispAttrName", sa.String(length=20), nullable=False),
        sa.Column("liveDbKey", sa.String(length=30), nullable=False),
        sa.Column("importKeyHash", sa.String(length=100), nullable=True),
        sa.Column("importGroupHash", sa.String(length=100), nullable=True),
        sa.Column("importDispHash", sa.String(length=100), nullable=True),
        sa.Column("propsJson", sa.String(length=500), nullable=True),
        sa.ForeignKeyConstraint(["coordSetId"], ["pl_diagram.ModelCoordSet.id"]),
        sa.ForeignKeyConstraint(
            ["dispId"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_DispKeyHash",
        "LiveDbDispLink",
        ["importKeyHash", "importDispHash", "dispAttrName"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_coordSetId",
        "LiveDbDispLink",
        ["coordSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_dispId",
        "LiveDbDispLink",
        ["dispId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_dispId_attr",
        "LiveDbDispLink",
        ["dispId", "dispAttrName"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_importGroupHash",
        "LiveDbDispLink",
        ["importGroupHash"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_liveDbUpdate",
        "LiveDbDispLink",
        ["dispId", "liveDbKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LiveDbDLink_liveKeyId",
        "LiveDbDispLink",
        ["liveDbKey"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispAction",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("propsJson", sa.String(length=500), nullable=True),
        sa.ForeignKeyConstraint(
            ["id"], ["pl_diagram.DispPolygon.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_table(
        "DispGroupItem",
        sa.Column("groupId", sa.Integer(), nullable=False),
        sa.Column("itemId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["groupId"], ["pl_diagram.DispGroup.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["itemId"], ["pl_diagram.DispBase.id"]),
        sa.PrimaryKeyConstraint("groupId", "itemId"),
        schema="pl_diagram",
    )
    op.create_table(
        "DispGroupPointer",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("rotation", sa.Integer(), server_default="0", nullable=False),
        sa.Column("verticalScale", sa.Float(), server_default="1.0", nullable=False),
        sa.Column("horizontalScale", sa.Float(), server_default="1.0", nullable=False),
        sa.Column("geomJson", sa.String(length=2000), nullable=False),
        sa.Column("groupId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["groupId"], ["pl_diagram.DispGroup.id"]),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxDispGroupPointer_groupId",
        "DispGroupPointer",
        ["groupId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispPolylineConn",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("connId", sa.Integer(), nullable=False),
        sa.Column("propsJson", sa.String(length=500), nullable=True),
        sa.ForeignKeyConstraint(["connId"], ["pl_diagram.ModelConn.id"]),
        sa.ForeignKeyConstraint(
            ["id"], ["pl_diagram.DispPolyline.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxConnCoordConnId",
        "DispPolylineConn",
        ["connId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_table(
        "DispGroupPointerNode",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("nodeId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["id"], ["pl_diagram.DispGroupPointer.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["nodeId"], ["pl_diagram.ModelNode.id"]),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )
    op.create_index(
        "idxNodeCoordNodeId",
        "DispGroupPointerNode",
        ["nodeId"],
        unique=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    raise NotImplementedError()



"""Renamed Location indexes

Peek Plugin Database Migration Script

Revision ID: 24b83c5e7e31
Revises: 23117803d414
Create Date: 2017-09-09 23:45:23.530049

"""

# revision identifiers, used by Alembic.
revision = "24b83c5e7e31"
down_revision = "23117803d414"
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    op.drop_index(
        "idx_DKIndexUpdate_gridKey",
        table_name="LocationIndexCompiled",
        schema="pl_diagram",
    )
    op.drop_index(
        "idx_DKIndexUpdate_coordSetId",
        table_name="LocationIndexCompiled",
        schema="pl_diagram",
    )
    op.drop_index(
        "idx_DKCompQueue_coordSetId_gridKey",
        table_name="LocationIndexCompilerQueue",
        schema="pl_diagram",
    )

    op.create_index(
        "idx_LIIndexUpdate_indexBucket",
        "LocationIndexCompiled",
        ["indexBucket"],
        unique=True,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LIIndexUpdate_modelSetId",
        "LocationIndexCompiled",
        ["modelSetId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_LICompQueue_modelSetId_indexBucket",
        "LocationIndexCompilerQueue",
        ["modelSetId", "indexBucket"],
        unique=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    raise NotImplementedError("Downgrade not implemented")



from datetime import datetime
from typing import Dict

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple

from peek_abstract_chunked_index.private.tuples.ACIUpdateDateTupleABC import (
    ACIUpdateDateTupleABC,
)


@addTupleType
class GridUpdateDateTuple(Tuple, ACIUpdateDateTupleABC):
    __tupleType__ = diagramTuplePrefix + "GridUpdateDateTuple"

    # Improve performance of the JSON serialisation
    __rawJonableFields__ = ("updateDateByChunkKey",)

    initialLoadComplete: bool = TupleField()
    updateDateByChunkKey: Dict[str, str] = TupleField()

    @property
    def ckiUpdateDateByChunkKey(self):
        return self.updateDateByChunkKey

    def ckiSetUpdateDateByChunkKey(self, value: Dict[str, str]) -> None:
        self.updateDateByChunkKey = value

    @property
    def ckiInitialLoadComplete(self) -> bool:
        return self.initialLoadComplete

    def ckiSetInitialLoadComplete(self, value: bool) -> None:
        self.initialLoadComplete = value



diagramFilt = {"plugin": "peek_plugin_diagram"}
diagramTuplePrefix = "peek_plugin_diagram."
diagramObservableName = "peek_plugin_diagram"
# This is an additional observable on the client, used to access the caches in the client
diagramActionProcessorName = "peek_plugin_diagram"
diagramTupleOfflineServiceName = "peek_plugin_diagram"



import logging
from typing import Union

from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.LookupCacheController import (
    LookupCacheController,
)

logger = logging.getLogger(__name__)


class ClientLookupTupleProvider(TuplesProviderABC):
    def __init__(self, lookupCacheController: LookupCacheController):
        self._lookupCacheController = lookupCacheController

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        return self._lookupCacheController.cachedVortexMsgBlocking(
            lookupTupleType=tupleSelector.name, filt=filt
        )



from typing import Dict
from typing import List
from typing import Optional

from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler
from vortex.handler.TupleDataObserverClient import TupleDataObserverClient

from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet


class CoordSetCacheController:
    """Lookup Cache Controller

    This class caches the lookups in each client.

    """

    def __init__(self, tupleObserver: TupleDataObserverClient):
        self._tupleObserver = tupleObserver
        self._tupleObservable = None

        #: This stores the cache of grid data for the clients
        self._coordSetCache: Dict[int, ModelCoordSet] = {}

        self._vortexMsgCache = None

    def setTupleObservable(self, tupleObservable: TupleDataObservableHandler):
        self._tupleObservable = tupleObservable

    def start(self):
        (
            self._tupleObserver.subscribeToTupleSelector(
                TupleSelector(ModelCoordSet.tupleName(), {})
            ).subscribe(self._processNewTuples)
        )

    def shutdown(self):
        self._tupleObservable = None
        self._tupleObserver = None
        self._coordSetCache = {}
        self._vortexMsgCache = None

    def _processNewTuples(self, coordSetTuples):
        if not coordSetTuples:
            return

        self._coordSetCache = {c.id: c for c in coordSetTuples}

        self._vortexMsgCache = None

        self._tupleObservable.notifyOfTupleUpdate(
            TupleSelector(ModelCoordSet.tupleName(), {})
        )

    @property
    def coordSets(self) -> List[ModelCoordSet]:
        return list(self._coordSetCache.values())

    def coordSetForId(self, coordSetId: int) -> Optional[ModelCoordSet]:
        return self._coordSetCache.get(coordSetId)

    def cachedVortexMsgBlocking(self, filt: dict) -> bytes:
        if self._vortexMsgCache:
            return self._vortexMsgCache

        data = self.coordSets

        # Create the vortex message
        vortexMsg = (
            Payload(filt, tuples=data).makePayloadEnvelope().toVortexMsg()
        )
        self._vortexMsgCache = vortexMsg
        return vortexMsg



"""added textHStretch

Peek Plugin Database Migration Script

Revision ID: 47879bec4c5f
Revises: a66e033b8227
Create Date: 2022-07-04 11:39:51.785436

"""

# revision identifiers, used by Alembic.
from sqlalchemy import Float

revision = "47879bec4c5f"
down_revision = "a66e033b8227"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispCurvedText",
        sa.Column(
            "textHStretch", Float, doc="hs", nullable=False, server_default="1"
        ),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column(
        "DispCurvedText",
        "textHStretch",
        schema="pl_diagram",
    )
    # ### end Alembic commands ###



from typing import Tuple

from vortex.Tuple import Tuple as VortexTuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class GridKeyTuple(VortexTuple):
    __tupleType__ = diagramTuplePrefix + "GridKeyTuple"

    gridKey: str = TupleField()

    width: float = TupleField()
    height: float = TupleField()

    shapeCount: int = TupleField()

    modelSetKey: str = TupleField()
    coordSetKey: str = TupleField()

    @property
    def gridXY(self) -> Tuple[int, int]:
        _, _, xy = self.gridKey.partition(".")
        x, _, y = xy.partition("x")

        x = int(x)
        y = int(y)

        return x, y



import json
import typing
from typing import Callable

from hashids import Hashids
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import Float
from sqlalchemy import ForeignKey
from sqlalchemy import Index
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy.orm import relationship
from vortex.Tuple import JSON_EXCLUDE
from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet
from peek_plugin_diagram._private.storage.ModelSet import ModelSet
from peek_plugin_diagram.tuples.ColorUtil import invertColor
from peek_plugin_diagram.tuples.lookup_tuples.ShapeColorTuple import (
    ShapeColorTuple,
)
from peek_plugin_diagram.tuples.lookup_tuples.ShapeLayerTuple import (
    ShapeLayerTuple,
)
from peek_plugin_diagram.tuples.lookup_tuples.ShapeLineStyleTuple import (
    ShapeLineStyleTuple,
)
from peek_plugin_diagram.tuples.lookup_tuples.ShapeTextStyleTuple import (
    ShapeTextStyleTuple,
)


class _Hasher:
    def __init__(self):
        self._hashids = Hashids(salt="7013b24ca9ff46188a1fbbb1fd0129e1")

    @property
    def encode(self) -> Callable:
        return self._hashids.encode

    @property
    def decode(self) -> Callable:
        return self._hashids.decode


_hasher = _Hasher()


@addTupleType
class DispLayer(Tuple, DeclarativeBase):
    __tablename__ = "DispLayer"
    __tupleTypeShort__ = "DLA"
    __tupleType__ = diagramTuplePrefix + __tablename__

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()
    modelSetKey = TupleField()

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(50), nullable=False)
    order = Column(Integer, nullable=False, server_default="0")
    selectable = Column(Boolean, nullable=False, server_default="false")
    visible = Column(Boolean, nullable=False, server_default="true")

    modelSetId = Column(
        Integer, ForeignKey("ModelSet.id", ondelete="CASCADE"), nullable=False
    )
    modelSet = relationship(ModelSet)

    importHash = Column(String(100), doc=JSON_EXCLUDE)

    showForEdit = Column(Boolean, nullable=False)

    blockApiUpdate = Column(Boolean, nullable=False)

    __table_args__: typing.Tuple = (
        Index("idx_DispLayer_modelSetId", modelSetId, unique=False),
        Index("idx_DispLayer_importHash", modelSetId, importHash, unique=True),
    )

    def setTupleFields(self) -> None:
        self.modelSetKey = self.modelSet.key
        self.data = {"modelSetKey": self.modelSet.key}
        self.key = _hasher.encode(self.id)

    def toTuple(self) -> "ShapeLayerTuple":
        self.setTupleFields()
        tuple_ = ShapeLayerTuple()

        tuple_.data = self.data
        tuple_.key = self.key
        tuple_.modelSetKey = self.modelSetKey

        tuple_.name = self.name
        tuple_.order = self.order
        tuple_.selectable = self.selectable
        tuple_.visible = self.visible
        tuple_.importHash = self.importHash
        tuple_.blockApiUpdate = self.blockApiUpdate
        tuple_.showForEdit = self.showForEdit

        return tuple_


@addTupleType
class DispLevel(Tuple, DeclarativeBase):
    __tablename__ = "DispLevel"
    __tupleTypeShort__ = "DLE"
    __tupleType__ = diagramTuplePrefix + __tablename__

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()
    modelSetKey = TupleField()
    coordSetKey = TupleField()

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(50), nullable=False)
    order = Column(Integer, nullable=False, server_default="0")
    minZoom = Column(Float)
    maxZoom = Column(Float)

    coordSetId = Column(
        Integer,
        ForeignKey("ModelCoordSet.id", ondelete="CASCADE"),
        nullable=False,
    )
    coordSet = relationship(ModelCoordSet, foreign_keys=[coordSetId])

    importHash = Column(String(100), doc=JSON_EXCLUDE)

    showForEdit = Column(Boolean, nullable=False)

    blockApiUpdate = Column(Boolean, nullable=False)

    __table_args__ = (
        Index("idx_DispLevel_coordSetId", coordSetId, unique=False),
        Index("idx_DispLevel_importHash", coordSetId, importHash, unique=True),
    )

    def setTupleFields(self) -> None:
        self.modelSetKey = self.coordSet.modelSet.key
        self.coordSetKey = self.coordSet.key
        self.data = {
            "modelSetKey": self.coordSet.modelSet.key,
            "coordSetKey": self.coordSet.key,
        }
        self.key = _hasher.encode(self.id)

    def isVisibleAtZoom(self, zoom: float) -> bool:
        return self.minZoom <= zoom < self.maxZoom

    def toTuple(self):
        self.setTupleFields()
        tuple_ = ShapeLayerTuple()

        tuple_.data = self.data
        tuple_.key = self.key
        tuple_.modelSetKey = self.modelSetKey
        tuple_.coordSetKey = self.coordSetKey

        tuple_.name = self.name
        tuple_.order = self.order
        tuple_.minZoom = self.minZoom
        tuple_.maxZoom = self.maxZoom
        tuple_.importHash = self.importHash
        tuple_.showForEdit = self.showForEdit
        tuple_.blockApiUpdate = self.blockApiUpdate

        return tuple_


@addTupleType
class DispTextStyle(Tuple, DeclarativeBase):
    __tupleTypeShort__ = "DTS"
    __tablename__ = "DispTextStyle"
    __tupleType__ = diagramTuplePrefix + __tablename__

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()
    modelSetKey = TupleField()

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(50), nullable=False)
    fontName = Column(String(30), nullable=False, server_default="GillSans")
    fontSize = Column(Integer, nullable=False, server_default="9")
    fontStyle = Column(String(30))
    scalable = Column(Boolean, nullable=False, server_default="true")
    scaleFactor = Column(Integer, nullable=False, server_default="1")
    spacingBetweenTexts = Column(
        Float, nullable=False, server_default="100", default=100
    )

    modelSetId = Column(
        Integer,
        ForeignKey("ModelSet.id", ondelete="CASCADE"),
        doc=JSON_EXCLUDE,
        nullable=False,
    )
    modelSet = relationship(ModelSet)

    importHash = Column(String(100), doc=JSON_EXCLUDE)

    borderWidth = Column(Float, nullable=True)

    showForEdit = Column(Boolean, nullable=False)

    blockApiUpdate = Column(Boolean, nullable=False)

    wrapTextAtChars = Column(Integer, nullable=True)

    __table_args__ = (
        Index("idx_DispTextStyle_modelSetId", modelSetId, unique=False),
        Index(
            "idx_DispTextStyle_importHash", modelSetId, importHash, unique=True
        ),
    )

    def setTupleFields(self) -> None:
        self.modelSetKey = self.modelSet.key
        self.data = {"modelSetKey": self.modelSet.key}
        self.key = _hasher.encode(self.id)

    def toTuple(self):
        self.setTupleFields()
        tuple_ = ShapeTextStyleTuple()

        tuple_.data = self.data
        tuple_.key = self.key
        tuple_.modelSetKey = self.modelSetKey

        tuple_.name = self.name
        tuple_.fontName = self.fontName
        tuple_.fontSize = self.fontSize
        tuple_.fontStyle = self.fontStyle
        tuple_.scalable = self.scalable
        tuple_.scaleFactor = self.scaleFactor
        tuple_.importHash = self.importHash
        tuple_.spacingBetweenTexts = self.spacingBetweenTexts
        tuple_.borderWidth = self.borderWidth
        tuple_.blockApiUpdate = self.blockApiUpdate
        tuple_.showForEdit = self.showForEdit

        return tuple_


@addTupleType
class DispLineStyle(Tuple, DeclarativeBase):
    __tupleTypeShort__ = "DLS"
    __tablename__ = "DispLineStyle"
    __tupleType__ = diagramTuplePrefix + __tablename__

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()
    modelSetKey = TupleField()

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(50), nullable=False)
    backgroundFillDashSpace = Column(
        Boolean, nullable=False, server_default="false"
    )
    capStyle = Column(String(15), nullable=False)
    joinStyle = Column(String(15), nullable=False)
    dashPattern = Column(String(50))
    startArrowSize = Column(Integer)
    endArrowSize = Column(Integer)
    winStyle = Column(Integer, nullable=False)

    modelSetId = Column(
        Integer,
        ForeignKey("ModelSet.id", ondelete="CASCADE"),
        doc=JSON_EXCLUDE,
        nullable=False,
    )
    modelSet = relationship(ModelSet)

    importHash = Column(String(100), doc=JSON_EXCLUDE)
    scalable = Column(Boolean, nullable=False, server_default="false")

    showForEdit = Column(Boolean, nullable=False)
    blockApiUpdate = Column(Boolean, nullable=False)

    __table_args__ = (
        Index("idx_DispLineStyle_modelSetId", modelSetId, unique=False),
        Index(
            "idx_DispLineStyle_importHash", modelSetId, importHash, unique=True
        ),
    )

    @property
    def makeData(self) -> dict:
        return {"modelSetKey": self.modelSet.key}

    @property
    def makeKey(self) -> str:
        return _hasher.encode(self.id)

    @property
    def dashPatternParsed(self):
        if self.dashPattern is None:
            return None
        return json.loads(self.dashPattern)

    def setTupleFields(self):
        self.modelSetKey = self.modelSet.key
        self.data = {"modelSetKey": self.modelSet.key}
        self.key = _hasher.encode(self.id)

    def toTuple(self):
        self.setTupleFields()
        tuple_ = ShapeLineStyleTuple()

        tuple_.data = self.data
        tuple_.key = self.key
        tuple_.modelSetKey = self.modelSetKey

        tuple_.name = self.name
        tuple_.backgroundFillDashSpace = self.backgroundFillDashSpace
        tuple_.capStyle = self.capStyle
        tuple_.joinStyle = self.joinStyle
        tuple_.dashPattern = self.dashPattern
        tuple_.startArrowSize = self.startArrowSize
        tuple_.endArrowSize = self.endArrowSize
        tuple_.winStyle = self.winStyle
        tuple_.importHash = self.importHash
        tuple_.scalable = self.scalable
        tuple_.showForEdit = self.showForEdit
        tuple_.blockApiUpdate = self.blockApiUpdate

        return tuple_


@addTupleType
class DispColor(Tuple, DeclarativeBase):
    __tupleTypeShort__ = "DC"
    __tablename__ = "DispColor"
    __tupleType__ = diagramTuplePrefix + __tablename__

    #: Misc data holder
    data = TupleField()

    #: Key field used to abstract ID for APIs with other plugins
    key = TupleField()
    modelSetKey = TupleField()

    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(50), doc=JSON_EXCLUDE, nullable=False)
    darkColor = Column(String(20), server_default="orange")
    lightColor = Column(String(20), server_default="orange")
    altColor = Column(String(20))
    swapPeriod = Column(Float)

    modelSetId = Column(
        Integer,
        ForeignKey("ModelSet.id", ondelete="CASCADE"),
        doc=JSON_EXCLUDE,
        nullable=False,
    )
    modelSet = relationship(ModelSet)

    importHash = Column(String(100), doc=JSON_EXCLUDE)

    showForEdit = Column(Boolean, nullable=False)

    blockApiUpdate = Column(Boolean, nullable=False)

    __table_args__ = (
        Index("idx_DispColor_modelSetId", modelSetId, unique=False),
        Index("idx_DispColor_importHash", modelSetId, importHash, unique=True),
    )

    def setTupleFields(self) -> None:
        self.modelSetKey = self.modelSet.key
        self.data = {"modelSetKey": self.modelSet.key}
        self.key = _hasher.encode(self.id)

    @property
    def color(self):
        return self.darkColor

    @color.setter
    def color(self, value: str):
        self.darkColor = value
        if value:
            self.lightColor = invertColor(value, "#fff")

    def toTuple(self):
        self.setTupleFields()
        tuple_ = ShapeColorTuple()

        tuple_.data = self.data
        tuple_.key = self.key
        tuple_.modelSetKey = self.modelSetKey

        tuple_.name = self.name
        tuple_.darkColor = self.darkColor
        tuple_.lightColor = self.lightColor
        tuple_.altColor = self.altColor
        tuple_.swapPeriod = self.swapPeriod
        tuple_.importHash = self.importHash
        tuple_.showForEdit = self.showForEdit
        tuple_.blockApiUpdate = self.blockApiUpdate
        tuple_.color = self.color

        return tuple_



from datetime import datetime
from typing import List
from typing import Tuple


from vortex.Tuple import Tuple as VortexTuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class DecodedCompiledGridTuple(VortexTuple):
    __tupleType__ = diagramTuplePrefix + "DecodedCompiledGridTuple"

    gridKey: str = TupleField()
    shapes: List[dict] = TupleField()
    lastUpdate: datetime = TupleField()

    isLinkedWithLookups: bool = TupleField(defaultValue=False)
    compiledShapes: List[dict] = TupleField(defaultValue=[])

    @property
    def gridXY(self) -> Tuple[int, int]:
        _, _, xy = self.gridKey.partition(".")
        x, _, y = xy.partition("x")

        x = int(x)
        y = int(y)

        return x, y
